/*
 * Buffer manager execute functions
 *
 *      version 2.0          21-Feb-1994     KEK     M.Nomachi
 *      debug for CDF CS     12-NOV-1993     KEK     Masaharu NOMACHI
 *          bug. on stimulate_producer()
 *            It could not handle more than "one" request.
 *          bug. on pause()
 *            Last consumer may back USED buffer to first consumer
 *            bm_done with negative significance will return the buffer
 *            to free buffer pool.
 *	version 1.02         30-SEP-1992	KEK	Masaharu NOMACHI
 *	                     16-JUL-1992	KEK	Masaharu NOMACHI
 *
 */

#include "bm.h"
#include "bm_exec.h"
#include "bm_config.h"

struct EXEC_process *consumer;
struct EXEC_process *wait_producer, *busy_producer;
struct EXEC_buffer  *free_buffer;
static int  threshold;

int exec_init(thr) int thr; {

  threshold = thr;

  /* Initialize the link top */
  free_buffer   = NULL_buffer;
  wait_producer = NULL_process;
  busy_producer = NULL_process;
  consumer      = NULL_process;
  
  return 0;
}

int exec_buff_install(table,offset,size)
     struct BMT_buffer *table; int offset, size;
{
  struct EXEC_buffer *buffer;
  
  /* Create buffer data base */
  buffer = (struct EXEC_buffer *)malloc(sizeof(struct EXEC_buffer));
  if ((int)buffer==-1) return -1;

  /* Initialize the table and data base */
  table->b_exec         = (char *)buffer;
  buffer->table         = table;
  buffer->type          = 0;
  buffer->significance  = 0;
  buffer->status        = BS_null;
  buffer->offset        = offset;
  buffer->max_size      = size;
  put_buffer(&free_buffer,buffer);

  return 0;
}

int exec_process_install(p_table,type,priority,n_request)
    struct BMT_process *p_table; int type, priority, n_request; {

  struct EXEC_process *p;
  struct EXEC_buffer  *b, *get_buffer();
  
  /* Create process data base */
  p = (struct EXEC_process *)malloc(sizeof(struct EXEC_process));
  if ((int)p == -1) return -1;
  
  /* Install it on the pointer table */
  p_table->p_exec = (char *)p;
  
  /* initialize the parameters */
  p->table      = p_table;
  p->type       = type;
  p->wait       = NULL_buffer;
  p->active     = NULL_buffer;
  p->n_request  = n_request;
  p->n_active   = 0;
  if (priority > 0) {
    p->priority   = priority;
    p->status     = PS_consumer;
    if (put_process(&consumer,p)) return -1;
  }
  else {
    p->priority   = -priority;
    p->status     = PS_wait;
    if (put_process(&wait_producer,p)) return -1;
    stimulate_producer();
  }
  return 0;
}

int exec_remove(p_table) struct BMT_process *p_table; {
  struct EXEC_process *p, *q;

  /* get the process removed */
  p = (struct EXEC_process *)(p_table->p_exec);
  
  /* remove it from the link */
  if      (p->status == PS_consumer) remove_process(&consumer,p);
  else if (p->status == PS_busy)     remove_process(&busy_producer,p);
  else if (p->status == PS_wait)     remove_process(&wait_producer,p);
  p_table->p_exec = (char *)0;

/* 29-Jan-2002, Bug fix.  E.Inoue */
/*  if ((p->status == PS_consumer) && (p->next != p)){ */
  if ((p->status == PS_consumer) && (p->next != consumer) && (p->next !=
p)){
    q = p->next;
    append_buffer(&(q->wait),p->active);
    append_buffer(&(q->wait),p->wait);
    stimulate_consumer(q);
  }
  else {
    append_buffer(&free_buffer,p->active);
    append_buffer(&free_buffer,p->wait);
    stimulate_producer();
  }
  /* Back it to free */
  free(p);

  return 0;
}

int bm_pause(p_table,control) struct BMT_process *p_table; char *control; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  

  if (p->status == PS_wait) {
    p->status = PS_busy;
    remove_process(&wait_producer,p);
    put_process(&busy_producer,p);
  }

  p->n_request = 0;

  if (control[0]=='d') {
#ifdef BMDEBUG1
    dump_process("pause %8X",p,(int)p->active);
#endif
    p->n_active = 0;
    if ((p->status == PS_consumer) && (p->next != consumer)){
      q = p->next;
      append_buffer(&(q->wait),p->active); 
      append_buffer(&(q->wait),p->wait); 
      p->active = NULL_buffer; 
      p->wait   = NULL_buffer; 
      stimulate_consumer(q);
    }
    else {
      append_buffer(&free_buffer,p->active); 
      append_buffer(&free_buffer,p->wait); 
      p->active = NULL_buffer; 
      p->wait   = NULL_buffer; 
      stimulate_producer();
    }
  }
  return 0;
}

int bm_resume(p_table,request) struct BMT_process *p_table; int request; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  
#ifdef BMDEBUG1
  dump_process("resume",p);
#endif

  p->n_request = request;
  append_buffer(&free_buffer,p->active);
  append_buffer(&free_buffer,p->wait);
  p->n_active  = 0;
  p->active = NULL_buffer;
  p->wait = NULL_buffer;

  if (p->status == PS_busy) {
    p->status = PS_wait;
    remove_process(&busy_producer,p);
    put_process(&wait_producer,p);
  }
  stimulate_producer();
  return 0;
}

int bm_request(p_table) struct BMT_process *p_table; {
  struct EXEC_process *p, *q;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  
#ifdef BMDEBUG1
  dump_process("request",p);
#endif

  if (p->n_request <= 0) p->n_request -= 1;
  return 0;
}

int bm_done(p_table,b_table,type,size,significance)
     struct BMT_process *p_table; struct BMT_buffer *b_table;
     int type, size, significance; 
{

  struct EXEC_process *p, *q, *c;
  struct EXEC_buffer  *b, *buff, *get_buffer();
  
  /* get the processes and the buffer */
  p = (struct EXEC_process *)(p_table->p_exec);
  b = (struct EXEC_buffer  *)(b_table->b_exec);

#ifdef BMDEBUG1
  dump_process("done %8X",p,(int)b);
#endif

  if (p->active != b) {
    printf("Invalid address\n");
    return BME_INVADR;
  }

  --(p->n_active);

  if (p->status == PS_consumer) {
    if (significance < 0) {
      forward_buffer(p,&free_buffer);
      stimulate_producer();
      return 0;
    }
    if (significance != 0) b->significance = significance;
    if (size > 0) {
      if (size > b->max_size) b->size = b->max_size;
      else                    b->size = size;
    }
    q = p->next;
    if ((q == consumer) || (p->n_request <= 0)) {
#ifdef BMDEBUG2
      if (q!=consumer) {
#ifndef BMDEBUG1
        dump_process("done",p);
#endif
        printf("**** A buffer is returned to pool");
        printf(" from not-last-process %8X\n",(int)p);
      }
#endif
      forward_buffer(p,&free_buffer);
      stimulate_producer();
    }
    else {
      forward_buffer(p,&(q->wait));
      stimulate_consumer(q);
    }
    stimulate_consumer(p);
  }
  /* Producer */
  else {
    b->significance = significance;
    if (size > b->max_size) b->size = b->max_size;
    else                    b->size = size;
    c = consumer;
    buff = get_buffer(&(p->active));
    if ((p->status == PS_busy) && (p->n_request > 0)) {
      p->status = PS_wait;
      remove_process(&busy_producer,p);
      put_process(&wait_producer,p);
    }

    /* Forward the buffer */
    if ((c!=NULL_process)&&(significance>0)) {
      put_buffer(&(c->wait),buff);
      stimulate_consumer(c);
    }
    else {
      put_buffer(&free_buffer,buff);
    }
    stimulate_producer();
  }
  return 0;
}

stimulate_consumer(p) struct EXEC_process *p; {
  struct EXEC_buffer  *buff, *get_buffer();
  struct EXEC_process *q;

#ifdef BMDEBUG1
  dump_process("stimulate",p);
#endif

  if (p->wait == NULL_buffer) return;
  if (p->n_request == 0) {
    q = p->next;
    if (q != consumer) {
      append_buffer(&(q->wait),p->wait);
      p->wait = NULL_buffer;
      stimulate_consumer(q);
    }
    else {
      append_buffer(&free_buffer,p->wait);
      p->wait = NULL_buffer;
      stimulate_producer();
    }
    return;
  }

  while ((p->n_active < p->n_request) || (p->n_request < 0)) {
    buff = get_buffer(&(p->wait));
    put_buffer(&(p->active),buff);
    if (bm_req_test(p->table,buff->table)) {
      (p->n_active) += 1;
      if (p->n_request < 0) p->n_request += 1;
#ifdef BMDEBUG1
  dump_process("send %8X",p,(int)buff);
#endif
      bm_send(p->table,buff->table,buff->offset,buff->size,buff->significance);
      buff->status = BS_sent;
    }
    else {
      buff->status = BS_passed;
    }
    if (p->wait == NULL_buffer) break;
    if (p->wait->significance + p->priority < threshold) break;
  }

  /* Forward the passed buffer */
  if (p->active != NULL_buffer) {
    q = p->next;
    if (q==consumer) {                                 
      if (p->active->status != BS_sent) forward_buffer(p,&free_buffer);
      stimulate_producer();
    }
    else {
      if (p->active->status != BS_sent) forward_buffer(p,&(q->wait));
      stimulate_consumer(q);
    }
  }
  return;
}
    
stimulate_producer() {
  struct EXEC_process *p, *c, *last;
  struct EXEC_buffer  *b, *get_buffer();

  if (wait_producer == NULL_process) return;
  last = wait_producer->previous;
  while(wait_producer != NULL_process) {
    /* get a free buffer */
    b = get_buffer(&free_buffer);
    if (b == NULL_buffer) {
      /* get from consumer processes */
      c = consumer;
      if (c == NULL_process) break;
      do {
	c = c->previous;
	b = c->wait;
	if (b != NULL_buffer) {
          if (b->next != b) {
            if ((b->significance + c->priority) < threshold) {
              b = get_buffer(&(c->wait));
#ifdef BMDEBUG2
#ifndef BMDEBUG1
              dump_process("stripped",p);
#endif
              printf("**** A buffer is stripped");
              printf(" from low priority process %8X\n",(int)p);
#endif
              break;
            }
	    else b = NULL_buffer;
          }
	  else b = NULL_buffer;
	}
      } while(c != consumer);
      if (b == NULL_buffer) break;
    }
    p = wait_producer;

#ifdef BMDEBUG1
   dump_process("stimulate",p);
#endif

    b->size = b->max_size;
    bm_send(p->table,b->table,b->offset,b->size,b->significance);
#ifdef BMDEBUG1
  dump_process("send %8X",p,(int)b);
#endif
    put_buffer(&(p->active),b);
    (p->n_active) += 1;
    if (p->n_active >= p->n_request) {
      remove_process(&wait_producer,p);
      put_process(&busy_producer,p);
      p->status = PS_busy;
    }
    else wait_producer = p->next;
/*
 * This line must be deleted so as more than two buffer queuing
 *
    if (last == p) break;
*/
  }
  return;
}
    
forward_buffer(p,destination)
     struct EXEC_process *p; struct EXEC_buffer **destination; 
{
  struct EXEC_buffer  *buff, *get_buffer();

  if (p->active == NULL_buffer) return;
  while(1) {
    buff = get_buffer(&(p->active));
    put_buffer(destination,buff);
    if (p->active == NULL_buffer) break;
    if (p->active->status != BS_passed) break;
  }
  return;
}

int bm_get_n_free() {
  int i;
  struct EXEC_buffer  *b;

  b = free_buffer;
  i = 0;
  while(i<30) {
    if (b == NULL_buffer) break;
    b = b->next;
    i++;
    if (b == free_buffer) break;
  }
  return i;
}

int bm_get_n_wait(p_table) struct BMT_process *p_table; {
  int i;
  struct EXEC_buffer  *b;
  struct EXEC_process *p;

  /* get the process */
  p = (struct EXEC_process *)(p_table->p_exec);
  if (p==NULL_process) return -1;
  if ((int)p == -1) return -1;

  b = p->wait;
  i = 0;
  while(i<30) {
    if (b == NULL_buffer) break;
    b = b->next;
    i++;
    if (b == p->wait) break;
  }
  return i;
}

#ifdef BMDEBUG1
#define BMDEBUG 1
#endif
#ifdef BMDEBUG2
#define BMDEBUG 1
#endif

#ifdef BMDEBUG
dump_process(string,p,var) char *string; struct EXEC_process *p; int var; {
  int n_wait, n_free;
  int bm_get_n_wait(), bm_get_n_free();


  if (p->status == PS_consumer) {
    n_wait = bm_get_n_wait(p->table);
    printf("BM-- PR(%4d)",p->priority);
    printf("WAIT:ACT=(%3d:%3d)",n_wait,p->n_active);
  }
  else {
    n_free = bm_get_n_free();
    printf("++BM PR(%4d)",p->priority);
    printf("FREE:ACT=(%3d:%3d)",n_free,p->n_active);
  }
  printf("  REQ(%2d)",p->n_request);
  printf("  ID(%08X)",(int)p);
  if (p->status == PS_consumer) printf(":consumer-- ");
  else                          printf(":producer++ ");
  printf(string,var);
  printf("\n");
  return 0;
}
#endif
