/*********************************************************
 * libcamlib.c          April 16 1993       v0.1         *
 *                                                       *
 * Author: C. Timmermans, University of Michigan         *
 *         Randall lab. of physics,                      *
 *         500 east University, Ann Arbor (MI), 48109    *
 * E-mail: timmer@mich.physics.lsa.umich.edu             *
 *                                                       *
 * Routines are similar to the routines written by       *
 * Y. Yasu, Online group, Physics department, KEK in     *
 * v0.8 of his camlib.c                                  *
 *********************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include "libcamlib.h"
#include "camlib.h"
#include<time.h>
#include<errno.h>
#include<stdio.h>
#include<fcntl.h>

#ifdef DEC
#include "vmelib.h"
#endif

#ifdef BIT3
#include<sys/file.h>
#include<sys/ioctl.h>
#include<sys/btio.h>
#ifdef MMAP
#include<sys/mman.h>
#include "set_admod.h"
#endif
#endif

#define MAXNLIST 50
#define MEMSIZE  0x2000

struct klist
{
  char name[9];
  short addr;
};

struct klist ldlist[MAXNLIST];
short  next_begin_addr;
int    n_list;
u_short k_commands[MEMSIZE];

extern int errno;

int cam;
short branch,crate;

#if defined(BIT3) && defined(MMAP)
int cam_path;
#endif

void kin_read(offset,data)
int offset;
u_short *data;
{
  u_short *addr,tmp;

#ifdef MMAP
  addr = (u_short *)(cam + offset);
  tmp = *addr;
#else
  lseek(cam, CAMAC_ADDR+offset, SEEK_SET);
  read(cam,&tmp,2);
#endif
#ifdef DEC
  *data = ((tmp & 0xff)<<8)+((tmp & 0xff00)>>8);
#else
  *data = tmp;
#endif
}

void kin_write(offset,data)
int offset;
u_short data;
{
  u_short *addr,tmp;

#ifdef DEC
  tmp = ((data & 0xff)<<8)+((data & 0xff00)>>8);
#else
  tmp = data;
#endif
#ifdef MMAP
  addr = (u_short *)(cam + offset);
  *addr = tmp;
#else
  lseek(cam, CAMAC_ADDR+offset, SEEK_SET);
  write(cam,&tmp,2);
#endif
}


int cam_open_( ) { return( COPEN( )); }
int camopn_( ) { return( COPEN( )); }
int copen_( ) { return( COPEN( )); }
int CAM_Open() { return( COPEN());}
int CAMOPN() { return( COPEN());}

int COPEN()
{
#ifdef BIT3
#ifdef MMAP
  caddr_t mmaddr;
  char *admod = ADMOD;
#endif
    bt_status_t     status_flags;
#endif

#ifdef DEC
    cam = (int)vme_mapopen(VME_A16D16MODE, CAMAC_ADDR,0x70);
    if(cam == -1) {
      if(errno != 0) return(errno);
      else           return(-50);
    }
#endif
#ifdef BIT3
    cam = open("/dev/bte64", O_RDWR);
    if (cam < 0) return(errno);
/*** Bring the adapter in a well known state ***/
    (void) ioctl(cam, BIOC_SETUP, &status_flags); 
/*** Make sure the Adaptor is connected ***/
    if (status_flags & BT_STATUS_MASK) return(-50); 
#ifdef MMAP
    cam_path = cam;
    mmaddr = mmap(NULL,0x70,(PROT_READ | PROT_WRITE),MAP_SHARED,cam,
		  CAMAC_ADDR);
    if(mmaddr == (caddr_t) -1)printf("Cannot mmap to remote address\n");
/*** Set the correct address modifier, requires group privileges ***/
    system(admod);
    cam = (int)mmaddr;
#endif
#endif
/*** The complete Initialization ***/
    kin_write(CSR,0x1000);
    kin_write(CSER,0);
    kin_write(DOCR,0x8010);
    kin_write(SCCR,0);
    kin_write(MTCR,0);
    kin_write(LAMICR,0x4);
    kin_write(DONICR,0x4);
    kin_write(EMPICR,0x4);
    kin_write(ABOICR,0x4);
    kin_write(LAMIVR,0xff);
    kin_write(DONIVR,0xff);
    kin_write(EMPIVR,0xff);
    kin_write(ABOIVR,0xff);
    kin_write(AMR,0x79);
    kin_write(CMAR,0x1fff);
    kin_write(CMR,0x80);
    return(CC_STA_OK);
}


int cam_close_( ) { return( CCLOSE( ) ); }
int camcls_( ) { return( CCLOSE( ) ); }
int cclose_( ) { return( CCLOSE( ) ); }
int CAM_Close( ) { return( CCLOSE( ) ); }
int CAMCLS( ) { return( CCLOSE( ) ); }

int CCLOSE()
{
#ifdef DEC
  vme_mapclose(CAMAC_ADDR,0x70);
#endif
#ifdef BIT3
#ifdef MEMMAP
  close(cam_path);
#else
  close(cam);
#endif
#endif
  return(CC_STA_OK);
}

/* NAF */
int naf_( n, a, f )
int *n, *a, *f; {
    return (NAFG( *n, *a, *f ));
}
int NAFG( n, a, f )
int n, a, f; {
    return  ((((n<<4) + a) << 5) + f);
}

int csetbr_( branch ) int *branch; { return( CSETBR(*branch) ); }
int CSETBR(i_br)
int i_br;
{
/****************************************************
 * branch is not used
 ****************************************************/
  if((i_br & 0x7) != i_br) return(-50);
  branch = i_br;
  return(CC_STA_OK);
}

int csetcr_( crate ) int *crate; { return( CSETCR(*crate) ); }
int CSETCR(i_cr)
int i_cr;
{
  if((i_cr & 0x7) != i_cr) return(-50);
  crate = i_cr;
  return(CC_STA_OK);
}

int cgenz_() { return( CGENZ() ); }
int CGENZ()
{
  int status,q,x = 0;
  short i = 0;

/*******************************************************************
 * Initializing the current CAMAC crate; check the CAMAC-master maual
 ********************************************************************/
/*** read status register ***/
  if((status = CAMACW(NAFGEN(30,0,1),&i,&q,&x)) != 0) return(status); 
/*** generate a Z == CAMAC initialize ***/
  i |=1;
  return(CAMACW(NAFGEN(30,0,17),&i,&q,&x));
}

int cgenc_() { return( CGENC() ); }
int CGENC()
{
  int status,q,x = 0;
  short i = 0;

/*********************************************************************
 * Clearing the dataway for current CAMAC-crate; check the CAMAC-master
 *********************************************************************/
/*** read status register ***/
  if((status = CAMACW(NAFGEN(30,0,1),&i,&q,&x)) != 0) return(status);
/*** generate a C == CAMAC clear ***/
  i |=2;
  return(CAMACW(NAFGEN(30,0,17),&i,&q,&x));
}

int cseti_() { return( CSETI() ); }
int CSETI()
{
  int status,q,x = 0;
  short i = 0;

/************************************************************************
 * set dataway inhibit of the current CAMAC crate; check the CAMAC-master
 ************************************************************************/
/*** read status register ***/
  if((status = CAMACW(NAFGEN(30,0,1),&i,&q,&x)) != 0) return(status);
/*** set the inhibit ***/
  i |=4;
  return(CAMACW(NAFGEN(30,0,17),&i,&q,&x));
}

int cremi_() { return( CREMI() ); }
int CREMI()
{
  int status,q,x = 0;
  short i = 0;

/***********************************************************************
 * reset dataway inhibit of current CAMAC crate; check the CAMAC-master
 **********************************************************************/
/*** read status register ***/
  if((status = CAMACW(NAFGEN(30,0,1),&i,&q,&x)) != 0) return(status);
/*** remove the inhibit ***/
  i &=~4;
  return(CAMACW(NAFGEN(30,0,17),&i,&q,&x));
}

int camacl_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMACL(*naf, data, q, x);
}
int camac_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMACL(*naf, data, q, x);
}

int CAMAC(naf,data,q,x)
int naf,*data,*q,*x;
{
  return(CAMACL(naf,data,q,x));
}

int CAMACL(naf,data,q,x)
int naf,*data,*q,*x;
{
  u_short low,high;
  int r_code;

  low = (*data & 0xffff);
  high = (*data >> 16) & 0xff;
  r_code = camac_pt(naf,&low,&high,q,x,0);
  *data = (low & 0xffff) + ((high & 0xff) << 16);
  return(r_code);
}

int camacw_( naf, data, q, x )
u_short *data;
int *naf, *q, *x; {
    return CAMACW( *naf, data, q, x );
}

int CAMACW(naf,data,q,x)
u_short *data;
int naf, *q, *x;
{
  u_short high;
  return(camac_pt(naf,data,&high,q,x,1));
}

int camac_pt(naf,low,high,q,x,ws)
int naf,ws,*q,*x;
u_short *low,*high;
{
  u_short tmp;

/*  printf("WS %d LOW %x HIGH %x\n",ws,*low,*high); */
/*** Reset the error bit ***/
  kin_write(CSR,0x1000);
  kin_write(CSER,0);
/*** set CMA register to beginning ***/
  kin_write(CMAR,0x1ffd); 
/*** Load data in command memory ***/
  kin_write(CMR,(crate << 8) + (ws << 1));
  kin_write(CMR,naf);
  kin_write(CMR,0x80);
/*** set CMA register to beginning ***/
  kin_write(CMAR,0x1ffd);
/*** Load the CSR bit 5 is set for write ***/
  if((naf & 0x18) == 0)
    kin_write(CSR,0x1);
  else 
    kin_write(CSR,0x21);
/*** wait until the ready/error bit is set ***/
  kin_read(CSR,&tmp);
/*  while((tmp & 0x1)) kin_read(CSR,&tmp); */
  if((naf & 0x8) == 0) while((tmp & 0x8100) == 0) kin_read(CSR,&tmp);
  if((naf & 0x18) == 0x10) {
/*** Write the data ***/
    kin_write(DLR,*low);
    if(ws == 0) kin_write(DHR,*high);
  }
  else if((naf & 0x8) == 0){
/*** Read the data ***/ 
    kin_read(DLR,low);
    if(ws == 0) kin_read(DHR,high);
  }
  kin_read(CSR,&tmp);
  if ((tmp & 0x2) == 0) *q = 1;
  else                  *q = 0;
  if ((tmp & 0x4) == 0) *x = 1;
  else                  *x = 0;
/*** Read the error-code ***/
  kin_read(CSR,&tmp);
  if((tmp&0x2000)) return(CC_STA_SINGLE_TIMEOUT);
  return(CC_STA_OK);
}

int cambloworwait_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBWWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int cbwwai_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBWWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int cdmaw_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBWWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int CDMAW(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
   return( CBWWAI(mode,naf,buffer,length,return_length,error));
}
int CamBloWorWAIt(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
   return( CBWWAI(mode,naf,buffer,length,return_length,error));
}
int CBWWAI(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
  return(camac_blpt(mode,naf,buffer,length,return_length,error,1));
}


int camblolonwait_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBLWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int cblwai_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBLWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int cdmal_( mode, naf, buffer, length, ret_len, err_code )
int *mode, *naf, *buffer, *length, *ret_len, *err_code; {
    return CBLWAI(*mode,*naf,buffer,*length,ret_len,err_code);
}
int CamBloLonWAIt(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
   return( CBLWAI(mode,naf,buffer,length,return_length,error));
}
int CDMAL(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
   return( CBLWAI(mode,naf,buffer,length,return_length,error));
}

int CBLWAI(mode,naf,buffer,length,return_length,error)
int mode,naf,*buffer,length,*return_length,*error;
{
  return(camac_blpt(mode,naf,buffer,length,return_length,error,0));
}

int camac_blpt(mode,naf,buffer,length,return_length,error,ws)
int mode,naf,*buffer,length,*return_length,*error,ws;
{
  u_short tmp,stat;

  *return_length = 0;
/*** Reset the error bit ***/
  kin_write(CSR,0x1000);
  kin_write(CSER,0);
/*** set CMA register to beginning ***/
  kin_write(CMAR,0x1ffb); 
/*** Load data in command memory ***/
  kin_write(CMR,0x20+(crate<<8) + (mode << 3) + (ws << 1));
  kin_write(CMR,naf);
  tmp = ((~length)&0xffff)+1;
  kin_write(CMR,tmp);
  kin_write(CMR,0xffff);
  kin_write(CMR,0x80);
/*** set CMA register to beginning ***/
  kin_write(CMAR,0x1ffb);
/*** Load the CSR bit 5 is set for write ***/
  if((naf & 0x18) == 0)
    kin_write(CSR,0x1);
  else 
    kin_write(CSR,0x21);
  kin_read(CSR,&stat);
/*  while((stat & 0x1)) kin_read(CSR,&stat); */
/*** wait until the ready/error bit is set ***/
  while((stat & 0x80) == 0) {
    if((naf & 0x8) == 0) while((stat & 0x8180) == 0) kin_read(CSR,&stat);
    if((stat&0x80) != 0) return(CC_STA_OK);
    if((naf & 0x18) == 0x10) {
/*** Write the data ***/
      kin_write(DLR,(*buffer&0xffff));
      if(ws == 0) kin_write(DHR,((*buffer&0xff0000) >> 16));
      buffer++;
      *return_length = *return_length + 1;
    }
    else if((naf & 0x8) == 0){
/*** Read the data ***/ 
      kin_read(DLR,&tmp);
      *buffer = tmp;
      if(ws == 0) {
        kin_read(DHR,&tmp);
        *buffer = *buffer + ((tmp&0xff)<<16);
      }
      buffer++;
      *return_length = *return_length + 1;
    }
    kin_read(CSR,&stat);
  }
  if ((stat & 0x2000) == 0)  return(CC_STA_OK);
  else                       return(CC_STA_SINGLE_TIMEOUT);
}


int cam_enablelam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int cenlam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int celam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int CAM_EnableLAM(mask)
int mask;
{
  return(CENLAM(mask));
}
int CELAM(mask)
int mask;
{
  return(CENLAM(mask));
}
int CENLAM(mask)
int mask;
{
  int q,x = 0;
  int i;

/*******************************************************************
 * Enables LAM on the current CAMAC crate; check the CAMAC-master maual
 ********************************************************************/
/*** Select LAM's ***/
  i = mask;
#ifdef NOVMEI
  return(CAMACL(NAFGEN(30,13,17),&i,&q,&x));
#else
  CAMACL(NAFGEN(30,13,17),&i,&q,&x);
  i = 1<<8;
  return(CAMACL(NAFGEN(30,0,17),&i,&q,&x));
#endif
}

int cam_waitlam_( timeout ) int *timeout; { return CWTLAM( *timeout );}
int cwtlam_( timeout )      int *timeout; { return CWTLAM( *timeout );}
int cwlam_( timeout )       int *timeout; { return CWTLAM( *timeout );}
int CAM_WaitLAM( timeout )  int  timeout; { return CWTLAM(  timeout );}
int CWLAM( timeout )        int  timeout; { return CWTLAM(  timeout );}
int CWTLAM(timeout)
int timeout;
{
  u_short i;
  long s;
  struct timeval to,t1,t2;
  struct timezone tz;

  to.tv_sec = 0;
  to.tv_usec = 500;  /* timeout is 500 usec */
  gettimeofday(&t1,&tz);
#ifdef NOVMEI
  kin_read(SRR,&i);
  while((i & (1<<crate)) == 0){
#else
  kin_read(LAMICR,&i);
  kin_write(LAMICR,i|0x10);
  kin_read(CSR,&i);
  while((i & 0x200) == 0){
#endif
    select(0,NULL,NULL,NULL,&to);
    gettimeofday(&t2,&tz);
    s = 1000000*(t2.tv_sec - t1.tv_sec) + t2.tv_usec - t1.tv_usec; 
    if(s >= (1000000*timeout)) break;
#ifdef NOVMEI
    kin_read(SRR,&i);
#else
    kin_read(CSR,&i);
#endif
  }
#ifdef NOVMEI
  kin_read(CSR,&i);
#endif
  if((i & 0x200) == 0)     return(1);
  else                     return(0);
}

int cam_disablelam_() { return CDSLAM();}
int cdslam_() {return CDSLAM();}
int cdlam_() {return CDSLAM();}
int CAM_DisableLAM() {return CDSLAM();}
int CDLAM() {return CDSLAM();}
int CDSLAM()
{
  int q,x = 0;
  int i;
  short j;

/*******************************************************************
 * Disables LAM on the current CAMAC crate; check the CAMAC-master maual
 ********************************************************************/
/*** Select LAM's ***/
  j = 0;
  CAMACW(NAFGEN(30,13,17),&j,&q,&x);
  i = 0;
  return(CAMACL(NAFGEN(30,0,17),&i,&q,&x));
}


/* CamReaLIST, CRLIST */
camrealist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
    return CRLIST(fname, list, *lenlis, retlen, err);
}
crlist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
    return CRLIST(fname, list, *lenlis, retlen, err);
}
CamReaLIST(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int lenlis, *retlen, *err;
{
    return CRLIST(fname, list, lenlis, retlen, err);
}
CRLIST(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int lenlis, *retlen, *err;
{
    FILE *fp;
    int size, a, b, c, d;

/*** CRLIST is copied from the KEK-library ***/
    if ((fp = fopen(fname, "r")) == NULL) {
        *err = errno;
        return errno;
    }
    (void)fscanf(fp, "%X", &size);
    if (size > lenlis) {
        *err = -1;
        return -1;
    }
    *retlen = size;
    while (size > 0) {
        (void)fscanf(fp, "%X%X%X%X", &a, &b, &c, &d);
        *list++ = a;
        *list++ = b;
        *list++ = c;
        *list++ = d;
        size -= 4;
    }
    *err = 0;
    return fclose(fp);
}

int ckinit_( ) { return( CKINIT( )); }
int CKINIT()
{
  next_begin_addr = 0;
  n_list = 0;
}

int ckadd_(name, list, lenlis )
char *name;
u_short *list;
int *lenlis; {
    return( CKADD( name, list, *lenlis )); }
int CKADD(name,list,length)
char *name;
u_short *list;
int length;
{
  u_short fnct,qmod,tmp;
  int index = 0;
  int ws,i_mem;

/*** Check for max. number of lists ***/
  if(n_list == MAXNLIST) return(CC_STA_KLIST_MEMORYFULL);
/*** Only use the first 8 characters, and check the name ***/
  if(strlen(name) >= 9) name[8] = '\0';
  for(tmp = 0 ; tmp < n_list; tmp ++) {
    if(strcmp(name,ldlist[tmp].name) == 0) return(CC_STA_KLIST_SAMENAME);
  }
/*** Add to the list, at least temporarily ***/
  strcpy(ldlist[n_list].name,name);
  ldlist[n_list].addr = next_begin_addr;
  n_list ++;

/*** Set the register to the beginning of the list ***/
  i_mem = next_begin_addr;
/*** Start to fill the register ***/
  while(index < (length -1)) {
/*** Check if we are overfilling the buffer ***/
    if(i_mem >= 0x1ff6) {
      n_list--;
      return(CC_STA_KLIST_MEMORYFULL);
    }
    fnct = list[index++] & 0xff;
/*    printf("List.... %x fnct %x Index %d\n",list[index-1],fnct,index-1); */
/*** 16 bits data transfer ***/
    if( (fnct == 0x1 ) || (fnct == 0x5 ) ||
        (fnct == 0x13) || (fnct == 0x17) ||
        (fnct == 0x1b) ) ws = 1;
/*** 24 bits transfer ***/
    else if( (fnct == 0x2 ) || (fnct == 0x6 ) ||
        (fnct == 0x14) || (fnct == 0x18) ||
        (fnct == 0x1c) ) ws = 0;
    else
                         ws = -1;
/*** Check for BLOCK transfer ***/
    if( (fnct == 0x17) || (fnct == 0x18))      qmod = 3;
    else if( (fnct == 0x1b) || (fnct == 0x1c)) qmod = 1;
    else                                       qmod = 0;
/*** Start filling: No data transfer at all ***/
    if(ws == -1){
      if(fnct == 0x9) {
	k_commands[i_mem++] = (crate << 8) + 1;
	k_commands[i_mem++] = list[index++];
      }
      else
	k_commands[i_mem++] = 0x80;
    }
    else{
/*** YES we are having data transfer ***/
      if((fnct == 1) || (fnct == 2)){
/*** a READ-function ***/
	k_commands[i_mem++] = (crate << 8) + (ws << 1) + 1;
	k_commands[i_mem++] = list[index++];
      }
      else if((fnct == 5) || (fnct == 6)){
/*** an inline write ***/
	k_commands[i_mem++] = (crate << 8) + 0x60 + (ws << 1) + 1;
	k_commands[i_mem++] = list[index++];
	k_commands[i_mem++] = list[index++];
	if(ws == 0) k_commands[i_mem++] = list[index++];
	else        k_commands[i_mem++] = 0;
      }
      else {
/*** a BLOCK-transfer ***/
	k_commands[i_mem++] = (crate << 8)+0x20+(qmod << 3)+(ws << 1)+1;
	k_commands[i_mem++] = list[index++];
	k_commands[i_mem++] = ((~list[index++])&0xffff)+1;
	k_commands[i_mem++] = 0xffff;
      }
    }
  }
  tmp = next_begin_addr;
  next_begin_addr = i_mem;
/*  printf("Next list will begin at address %x\n",next_begin_addr); */
/*** Really write it in the K2917 ***/
  kin_write(CMAR,tmp);
  for(i_mem = tmp; i_mem < next_begin_addr; i_mem++) 
    kin_write(CMR,k_commands[i_mem]);
  return(CC_STA_OK);
}

int ckexec_(name, buf, lenbuf, iiosb )
char *name;
u_short *buf;
int *lenbuf;
struct cc_iosb *iiosb; {
    return( CKEXEC( name, buf, *lenbuf, iiosb ) ); }
int CKEXEC(name,buffer,length,iosb)
char *name;
u_short *buffer;
int length;
struct cc_iosb *iosb;
{
  u_short addr = 0x3000;
  u_short tmp,stat,i;
  u_short *dev_tmp;
  u_short ws;

  iosb->ret_length = 0;
  dev_tmp = (u_short *)(&(iosb->devinfo));
  for(tmp = 0 ; tmp < n_list; tmp ++) {
    if(strncmp(name,ldlist[tmp].name,strlen(ldlist[tmp].name)) == 0) 
      addr = ldlist[tmp].addr;
  }
  iosb->status = 1;
/*** reset the K2917 ***/
  kin_write(CSR,0x1000);
/*  printf("addr ... %x\n",addr); */
  if(addr == 0x3000) return(CC_STA_KLIST_NONAME);
/*** Set the register to the beginning of the list ***/
  kin_write(CMAR,addr);
/*  for(i=0;i< next_begin_addr; i++){
    printf ("Command.... %x\n",k_commands[i]);
  } */
  kin_write(CSR,1);
/*** wait until the ready/error bit is set ***/
  kin_read(CSR,&stat);
/*  while((stat & 0x1)) kin_read(CSR,&stat); */
  while((stat & 0x80) == 0) {
    while((stat & 0x8180) == 0) kin_read(CSR,&stat);
    if((stat&0x80) != 0) {
      dev_tmp[0] = stat&0xffff;
      kin_read(CSER,&tmp);
      dev_tmp[1] = tmp&0x1f;
      return(CC_STA_OK);
    }
    else if ((stat & 0x100) != 0) {
/*** I have to read data, but 24 or 16 bits, check the command ***/
      ws = 8;
      while(ws == 8) {
        if((tmp = (k_commands[addr] & 0xe0)) == 0x60) addr+=4;
        else if ((tmp & 0xe0) == 0x80) {
	  printf("Bad list\n");
	  dev_tmp[0] = stat&0xffff;
	  kin_read(CSER,&tmp);
	  dev_tmp[1] = tmp&0x1f;
	  return(-1);
	}
        else {
	  if((k_commands[addr+1] & 0x18) != 0)        addr+=2;
	  else {
	    ws = (k_commands[addr] & 6) >> 1;
	    if((k_commands[addr] & 0x60) == 0) addr+=2;
	    else{
	      kin_read(CWCR,&tmp);
	      tmp = ((~tmp)&0xffff)+1;
	      if(tmp == 0) addr+=4;
	    }
	  }
	}
      }
/*** The ws is known, and the addr-pointer is set ***/
      kin_read(DLR,&(buffer[iosb->ret_length]));
      if(ws == 0) {
	kin_read(DHR,&tmp);
	buffer[iosb->ret_length]+=((tmp&0xff)<<16);
      }
      iosb->ret_length+=1;
    }
    kin_read(CSR,&stat);
  }
  if ((stat & 0xc000) == 0)  {
    dev_tmp[0] = stat&0xffff;
    kin_read(CSER,&tmp);
    dev_tmp[1] = tmp&0x1f;
    return(CC_STA_OK);
  }
  printf("Bad state\n");
  dev_tmp[0] = stat&0xffff;
  kin_read(CSER,&tmp);
  dev_tmp[1] = tmp&0x1f;
  return(-1);
}

int cksave_(fname)
char *fname; {
    return( CKSAVE( fname ) ); }
int CKSAVE(name)
char *name;
{
  short bf[12000];
  short lsize,size;
  int i,fd;

/*** Offset to store the total size ***/
  size = 1;
/*  printf("Storing %d lists\n",n_list); */
  for(i = 0;i<n_list;i++) {
/*** Store the name of the list ***/
/*    printf("address %x name %s\n",ldlist[i].addr,ldlist[i].name); */
    strcpy(&(bf[size]),ldlist[i].name);
    size+=((1+sizeof(ldlist[i].name))/2);
/*** Get the size of the list ***/
    if(i == (n_list - 1)) lsize = (next_begin_addr - ldlist[i].addr);
    else                  lsize = (ldlist[i+1].addr - ldlist[i].addr);
/*** Store the size of the list ***/
    bf[size++] = lsize;
/*** Store the list ***/
    memcpy(&(bf[size]),&(k_commands[ldlist[i].addr]),2*lsize);
    size+=lsize;
  }
/*** Store the total size - the size of the first word ***/
  bf[0] = size-1;
/*** Write it all to file ***/
  fd = creat(name,0666);
  write(fd,&(bf[0]),2);
  write(fd,&(bf[1]),2*bf[0]);
  close(fd);
  return(0);
}

int ckload_(fname)
char *fname; {
    return( CKLOAD( fname ) ); }
int CKLOAD(name)
char *name;
{
  short bf[12000];
  short lsize,size;
  int i,fd;

/*  printf("Loading a list\n"); */
  fd = open(name,O_RDONLY);
  if(fd == -1) return(errno);
  read(fd,&(bf[0]),2);
  read(fd,&(bf[1]),2*bf[0]);
  close(fd);
/*** First load it all in the library structures ***/
  n_list = 0;
  next_begin_addr = 0;
  i = 1;
  while(i < bf[0]) {
    ldlist[n_list].addr = next_begin_addr;
    strcpy(ldlist[n_list].name,&(bf[i]));
/*  printf("address %x name %s\n",ldlist[n_list].addr,ldlist[n_list].name); */
    i+=((1+sizeof(ldlist[n_list].name))/2);
    lsize = bf[i++];
    memcpy(&(k_commands[next_begin_addr]),&(bf[i]),2*lsize);
    n_list ++;
    next_begin_addr+=lsize;
    i+=lsize;
  }
/*  printf("Loaded %d lists, next address %x\n",n_list,next_begin_addr); */
/*** Now load it all in the K2917 ***/
  kin_write(CMAR,0);
  for(i = 0; i < next_begin_addr; i++) 
    kin_write(CMR,k_commands[i]);
  return(CC_STA_OK);
}
