#!/bin/csh -f
#--------------------------------------------------------------------------#
#	This is a c-shell script.  It should be executed once, when        #
#	UNIDAQ is first unwound from the tar files, to set up the machine  #
#	specific variables needed by the UNIDAQ system to correctly run.   #
#	Default values are supplied in the prompts.  Output of this script #
#	is the file       setup.csh      which can be sourced from the     #
#	user's .cshrc file in order to set up the needed UNIDAQ            #
#	environment plus a few other files.  See the settings of the       #
#       "fname" variable for other files which are set up.                 #
#                                                                          #
#       If a single argument of arbitrary value is supplied to this script #
#       it is assumed that daqlndir was used to make a complete (link)     #
#       copy of a UNIDAQ tree, so only those settings which are absolutely #
#       necessary are remade.  The directory which was copied must have a  #
#       valid setup.csh file in the usual place.                           #
#                                                                          #
#       Modified 11/17/94, R.Ball, for camvme(HP) installation and         #
#       multiple analyzer versions.                                        #
#--------------------------------------------------------------------------#

echo "-----------------------------------------------------------------------"
echo "             ==> Installation setup script for UNIDAQ <=="
set nargs = $#argv
if ( $#argv != 0 ) then
   echo "            ==>  Special version for use with daqlndir  <=="
endif

#	Set up the local variables.  Start with TOPUNIDAQ.

echo "Variables will now be set.  Defaults are inside [ ]."
echo "Enter a <CR> to accept the default."
echo ""

baddir:
set pd=`pwd`
set pd2=$pd
echo "Setting UNIDAQ root directory."
echo -n "TOPUNIDAQ = [$pd] "
set ans=$<
if ( $ans == "" ) then
else
   set pd=$ans
   set pd2="`echo $ans`"
endif
echo ""
if ( ! -e $pd2/install.csh ) then
   echo "$pd is not an acceptable directory.  Please try again."
   goto baddir
endif

#	With TOPUNIDAQ now known, cd to it and begin to write out the
#	setup files.  Make sure that g+w is available on all files in the
#	tree.

cd $pd2
chmod -R g+w . >&/dev/null

set fname=conf/setup.csh

if ( -e $fname ) then
  set hallo = `cat $fname | grep BVME`
  set hallm = `cat $fname | grep MURMUR_SERVER_ADD`
else
  if ( $nargs != 0 ) then
    echo "          ==>  Revert to full install (sources only found) <=="
    set nargs = 0
  endif
  set hallo = "setenv BVMELIB "
  set hallm = ""
endif

rm -f $fname
rm -f setup.csh
ln -s $fname setup.csh

#	Save this first file name so that the MURMUR_SERVER_ADD can be
#	added to its end.

set savename=$fname

#	Write out the file header

cat >$fname <<ZZZ
#!/bin/csh
#---------------------------------------------------------------#
#          Setup file for UNIDAQ environment variables.         #
#          This file is named "setup.csh" and is                #
#          located in the conf directory of the UNIDAQ          #
#	   system.  A link to this file by the same name is     #
#	   provided in the top level directory.                 #
#          Source this file once at login, or from your .cshrc  #
#          file.                                                #
#---------------------------------------------------------------#

ZZZ

echo '# Setting aliases'	>>$fname
echo 'alias @env-var-rm\'	>>$fname
echo  \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\ 	>>$fname
echo  \''setenv \\!:1 `echo :${\\!:1} | sed -e s~:\\!:2~~g -e s~^:~~`;'\'\\ >>$fname
echo  \''if ("${\\!:1}" == "")  unsetenv \\!:1;'\'\\ >>$fname
echo "" >>$fname

echo 'alias @env-var-prepend\'			>>$fname
echo  \''@env-var-rm \\!:1 \\!:2;'\'\\	>>$fname
echo  \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\		>>$fname
echo  \''if ("${\\!:1}" != "")  setenv \\!:1 :"$\\!:1";'\'\\	>>$fname
echo  \''setenv \\!:1 "\\!:2$\\!:1"'\'				>>$fname
echo ""	>>$fname
    
echo 'alias @env-var-append\'			>>$fname
echo \''@env-var-rm \\!:1 \\!:2;'\'\\		>>$fname
echo \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\		>>$fname
echo \''if ("${\\!:1}" != "")  setenv \\!:1 "$\\!:1": ;'\'\\	>>$fname
echo \''setenv \\!:1 "${\\!:1}"\\!:2'\'\\			>>$fname
echo "" >>$fname

echo 'alias @path-rm'  \''set path = (`echo " $path " | sed "s~ \\!:1 ~ ~g"`)'\'>>$fname
echo 'alias @path-prepend'  \''@path-rm \\!:1; set path = (\\!:1 $path)'\'	>>$fname
echo 'alias @path-append'   \''@path-rm \\!:1; set path = ($path \\!:1)'\'	>>$fname

echo ""	>>$fname

echo '# Remove old UNIDAQ PATH and MANPATH traces, if any' >>$fname
echo 'if ( ${?TOPUNIDAQ} ) then' >>$fname
echo '    @env-var-rm MANPATH $TOPUNIDAQ/man' >>$fname
echo '    @path-rm $TOPUNIDAQ/bin' >>$fname
echo '    @path-rm $TOPUNIDAQ/NOVA/src' >>$fname
echo 'endif' >>$fname
echo ""	>>$fname

echo '# Now set the variables for this version of UNIDAQ' >>$fname
echo "setenv TOPUNIDAQ          $pd" >>$fname
echo "setenv USERUNIDAQ         $pd" >>$fname
echo "" >>$fname
echo "setenv UNIDAQversion      2.3" >>$fname
echo "" >>$fname
set pdsave=$pd
unset pd

#	Define the machine type.

machine_choice:
set ffname=`uname -s`
echo ""
echo "Is this installation for:"
echo "   $ffname   = 1"
echo "   VXWORKS   = 2"
if ( $ffname == HP-UX ) then
  echo "   HP-RT     = 3"
endif
echo -n "      Make your choice now [1] "
set ans=$<
if ( $ans == "" ) then
  set ans=1
endif
if ( ! ( $ans == "1" || $ans == "2" || $ans == "3" ) ) then
  echo "           Choice is not acceptible, try again."
  goto machine_choice
endif

if ( $ans == "1" ) then
  if ($ffname == SunOS) then
     setenv MACHINE SUN
     echo "setenv MACHINE            SUN" >>$fname
  else if ($ffname == HP-UX) then
     setenv MACHINE HP
     echo "setenv MACHINE            HP" >>$fname
  else if ($ffname == ULTRIX) then
     setenv MACHINE DEC
     echo "setenv MACHINE            DEC" >>$fname
  else if ($ffname == IRIX) then
     setenv MACHINE SGI
     echo "setenv MACHINE            SGI" >>$fname
  endif
else if ( $ans == "2" ) then
  setenv MACHINE VXWORKS
  echo "setenv MACHINE            VXWORKS" >>$fname
else
  setenv MACHINE HPRT
  echo "setenv MACHINE            HPRT" >>$fname
endif
unset ffname


#	Set up the CERN_ROOT variable.

if ( $MACHINE == "VXWORKS" || $MACHINE == "HPRT" ) then
else
  set dcerntst=`printenv|grep CERN_ROOT`
  if ( $#dcerntst == 0 ) then
     set cernroot=""
  else
     set cernroot=`echo $CERN_ROOT`
  endif
  unset dcerntst

  echo "Setting CERN library root."
  echo -n "CERN_ROOT = [$cernroot] "
  set ans=$<
  if ( $ans == "" ) then
  else
    set cernroot=$ans
  endif

  echo "setenv CERN_ROOT          $cernroot" >>$fname
  unset cernroot
  echo ""

#	Some 'fixed' variables.

  echo 'setenv MURMUR_CLIENT_DIR  $TOPUNIDAQ/murmur_client/current' >>$fname
#  echo 'setenv MURMUR_DIR         $TOPUNIDAQ/murmur_dir' >>$fname
  echo "" >>$fname
endif


#	put out the machine specific compilation variables.

if ( $MACHINE == SUN ) then 
   echo "setenv X11LIB             /usr/lib/X11/libX11.a" >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11 " >>$fname
   echo "setenv LDFLAGS            -Bstatic " >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77" >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
#   echo 'setenv CXX               "CC -D_CPP"' >>$fname
#   echo 'setenv LD                "CC"' >>$fname
#   set thiscc=CC
else if ( $MACHINE == HP ) then
   echo 'setenv X11LIB             "-L/usr/lib/X11R5 -lX11"' >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11R5 " >>$fname
   echo "setenv LDFLAGS            -lBSD  " >>$fname
   echo 'setenv F77FLAGS           "+ppu"' >>$fname
   echo "setenv FF                 fort77" >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
#   echo 'setenv CXX               "CC -D_CPP"' >>$fname
#   echo 'setenv LD                "CC"' >>$fname
#   set thiscc=CC
else if ( $MACHINE == DEC ) then
   echo "setenv X11LIB             /usr/lib/libX11.a" >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11" >>$fname
   echo 'setenv LDFLAGS            ""' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77 " >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
#   echo 'setenv CXX               "g++ -D_CPP -fno-strict-prototype"' >>$fname
#   echo 'setenv LD                "g++"' >>$fname
#   set thiscc=g++
else if ( $MACHINE == SGI ) then
   echo 'setenv X11LIB             "-lX11 -lXt -lXmu"' >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11" >>$fname
   echo 'setenv LDFLAGS            "-lc"' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77 " >>$fname
   echo "setenv SUN_EXTLIB         -lsun " >>$fname
#   echo 'setenv CXX               "g++ -D_CPP -fno-strict-prototype"' >>$fname
#   echo 'setenv LD                "g++"' >>$fname
#   set thiscc=g++
else if ( $MACHINE == HPRT ) then
   echo "setenv CC                 ccrt" >>$fname
   echo 'setenv X11LIB             "-L$HPRTroot/usr/lib/X11R5 -lX11"' >>$fname
   echo 'setenv CX11FLAGS          "-DXLIB_ILLEGAL_ACCESS -I$HPRTroot/usr/include"' >>$fname
   echo 'setenv LDFLAGS            "-L$HPRTroot/lib -lbsd -lrpc"' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77rt" >>$fname
   echo 'setenv SUN_EXTLIB         ""' >>$fname
#   echo 'setenv CXX               "CCrt -D_CPP"' >>$fname
#   echo 'setenv LD                "CCrt"' >>$fname
#   set thisCC=CCrt
else if ( $MACHINE == VXWORKS ) then
   echo "setenv CC                 cc68k" >>$fname
   echo "setenv LD                 ld68k" >>$fname
   echo "" >>$fname

   set dvw=`printenv|grep VWROOT`
   if ( $#dvw == 0 ) then
     set vwroot=""
   else
     set vwroot=$VWROOT
   endif
   unset dvw
   echo -n "Enter VxWorks root directory [$vwroot] "
   set ans=$<
   if ( $ans == "" ) then
   else
     set vwroot=$ans
   endif
   echo "setenv VWROOT             $vwroot" >>$fname
   unset vwroot

   echo -n "Enter VxWorks CPU-type [MC68040] "
   set ans=$<
   if ( $ans == "" ) then
     set ans="MC68040"
   endif
   echo "setenv CPUFLAG            -DCPU=$ans" >>$fname

   set vxename=conf/vxenvvar.h

   rm -f $vxename
   touch $vxename
   echo "/* This file contains the VXWORKS env. variables */" >>$vxename
   echo "int n_vx_env_var = 2;"           >>$vxename
   echo " "                               >>$vxename
   echo "struct envvar {"                 >>$vxename
   echo "char *name;"                     >>$vxename
   echo "char *value;"                    >>$vxename
   echo "};"                              >>$vxename
   echo " "                               >>$vxename
   echo "struct envvar vx_env_var[] = { " >>$vxename
   echo -n '"TOPUNIDAQ","'                >>$vxename
   echo -n "$pdsave"                      >>$vxename
   echo '",'                              >>$vxename
   echo -n '"USERUNIDAQ","'               >>$vxename
   echo -n "$pdsave"                      >>$vxename
   echo '",'                              >>$vxename
   echo "};"                              >>$vxename
   touch $vxename
endif

#	Following sections will only be run if this is an original install.
#	If the installation is done through 'daqlndir', then just copy
#	the stuff from the original installation.  Determination of 'daqlndir'
#	installation is made if any arguments appears on the
#	command line.

#	For VXWORKS, set the Makefile correctly, set the camvme link, and 
#	skip down 4 pages or so within this file.

#	For NOT VXWORKS:
#	Determine which camac driver software to use.  This requires 
#	copying one of the two Makefile templates (.camac or .nocamac) as the
#	correct Makefile in the lib directory.  It may also require changing
#	the machine-specific links to the camac libraries.
#	One more machine-dependent compilation variable will also be put out.
#	The default is Makefile.camac .

	if ( $nargs == 0 ) then
cd lib
rm -f Makefile
if ( $MACHINE == VXWORKS ) then
   cp Makefile.camac Makefile
   cd ../$MACHINE/vme
   if ( ! -e camac.distribute ) then
     mv camac camac.distribute
   else
     rm -f camac
   endif
   ln -s ../../camvme/vme/camac camac
   cd ../..
   unset fname
   goto nodefile
endif

cp Makefile.camac Makefile
camac_choice:
echo "Setting camac/vme library directory. "
if ( $MACHINE == SUN ) then 
   echo "   UNIDAQ distribution (v 1.22d)   = 1"
   echo "   Your own local directory       = 2"
   echo -n "      Make your choice now [1]      "
   set ans=$<
   if ( $ans == "" ) then
      set ans=1
   endif
   if ( ! ( $ans == "1" || $ans == "2" ) ) then
      echo "      Choice is not acceptable."
      goto camac_choice
   endif
   echo 'setenv BVMELIB            "" ' >>../$fname
else if ( $MACHINE == HP ) then
   echo "   Bit3 library with Bit3 v1.7    = 3"
   echo -n "      Make your choice now [3]      "
   set ans=$<
   if ( $ans == "" ) then
      set ans=3
   endif
   if ( ! ( $ans == "3" ) ) then
      echo "      Choice is not acceptable."
      goto camac_choice
   endif
   echo 'setenv BVMELIB            "" ' >>../$fname
#
# See if CAMAC should be build using the MMAP option.
#
   cat <<ZZZ
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 The standard camac distribution for UNIDAQ treats CAMAC as a slow device
 and does seek/read or seek/write pairs for operations.  A faster way uses
 memory mapping, but requires root access for one image.  If you want faster
 CAMAC access by a factor of approximately 65, you can build the libraries 
 using memory mapping, and then when the full make is completed, manually 
 (as root) change the  protection bits for the set_admod file.  The command 
 sequence to do this is, as root:
       cd HP/vme/camac/etc
       chmod 4755 set_admod
       cd ../../../..
 If you wish to use memory mapping, answer MM to the following question,
 otherwise just hit the CARRIAGE-RETURN key.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
ZZZ
   set a1=0
   while ( $a1 == "0" )
     echo ""
     echo -n " Standard (default) or Memory Mapped CAMAC [<cr>|MM] "
     set answ=$<
     if ( $answ == "" ) then
        set a1=1
     else if ( $answ == "MM" || $answ == "mm" ) then
        set a1=2
     endif
     if ( $a1 == "0" ) then
        echo "Please enter a carriage return or MM"
     endif
   end
   unset answ
#
#  The remainder of the setups for the CAMAC lib are in the next section of
#  this file.
#
else if ( $MACHINE == DEC ) then
   echo "   UNIDAQ distribution (v 1.01)    = 1"
   echo "   Your own local directory       = 2"
   echo "   camvme library with DEC turbo  = 3"
   echo -n "      Make your choice now [1]      "
   set ans=$<
   if ( $ans == "" ) then
      set ans=1
   endif
   if ( ! ( $ans == "1" || $ans == "2" || $ans == "3" ) ) then
      echo "      Choice is not acceptable."
      goto camac_choice
   endif
   echo setenv BVMELIB            \'-L\$\(MAINDIR\)/DEC/vme/lib -lvmelib\' >>../$fname
else if ( $MACHINE == SGI ) then
   echo "   UNIDAQ distribution            = 1"
   echo -n "      Make your choice now [1]      "
   set ans=$<
   if ( $ans == "" ) then
      set ans=1
   endif
   if ( ! ( $ans == "1" ) ) then
      echo "      Choice is not acceptable."
      goto camac_choice
   endif
   echo 'setenv BVMELIB            "" ' >>../$fname
else if ( $MACHINE == HPRT ) then
   echo "   UNIDAQ distribution (v1.02)    = 1"
   echo -n "      Make your choice now [1]      "
   set ans=$<
   if ( $ans == "" ) then
      set ans=1
   endif
   if ( ! ( $ans == "1" ) ) then
      echo "      Choice is not acceptable."
      goto camac_choice
   endif
   echo 'setenv BVMELIB            "" ' >>../$fname
endif
cd ..

cd $MACHINE/vme
camac_dir:
if ( $ans == "2") then
   set pd=/usr/local/vme/camac
   echo -n " Enter the top camac directory [$pd] "
   set ant=$<
   if ( $ant == "" ) then
   else
      set pd=$ant
   endif
   unset ant
   if ( ! -e $pd ) then
      goto camac_dir
   endif
   if ( ! -e camac.distribute ) then
     mv camac camac.distribute
   else
     rm -f camac
   endif
   ln -s $pd camac
   unset pd
#	Modify the makefile to completely ignore the camac library 
   rm -f ../../lib/Makefile
   cp ../../lib/Makefile.nocamac ../../lib/Makefile
else if ( $ans == "3" ) then
   if ( ! -e camac.distribute ) then
     mv camac camac.distribute
   else
     rm -f camac
   endif
   ln -s ../../camvme/vme/camac camac
#
# following is the remainder of the CAMAC lib setups from the previous section
# of this file.
#
   if ( $MACHINE == HP ) then
    if ( $a1 == "1" ) then
     cd camac/lib
     \rm Makefile
     cp Makefile.standard Makefile
     cd ../..
    else
     echo " Be SURE to issue the chmod command above after the make completes"
     echo " ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
     set lgb="$pdsave/HP/vme/camac/etc/set_admod"
     cd camac/lib
     \rm Makefile
     cp Makefile.mmap Makefile
     cd ../include
     \rm set_admod.h
     cd ../..
     set lzz='"'
     echo "#define ADMOD $lzz$lgb$lzz" > $pdsave/HP/vme/camac/include/set_admod.h
     unset lgb
     unset lzz
    endif
    unset a1
   endif
else if ( -e camac.distribute ) then
   rm -f camac
   mv camac.distribute camac
endif
cd ../..
	else
echo $hallo >>$fname
	endif

echo "" >>$fname
echo ""

echo '# Set the MANPATH and PATH correctly for this UNIDAQ version' >>$fname
echo '@path-prepend $TOPUNIDAQ/bin'	>>$fname
echo '@path-prepend $TOPUNIDAQ/NOVA/src' >>$fname
echo 'set dmantst=`printenv|grep MANPATH`' >>$fname
echo 'if ( $#dmantst > 0 ) then' >>$fname
echo '  @env-var-prepend MANPATH $TOPUNIDAQ/man' >>$fname
echo 'else' >>$fname
echo '  setenv MANPATH	  $TOPUNIDAQ/man:/usr/man' >>$fname
echo 'endif' >>$fname
echo 'unset dmantst' >>$fname
echo 'rehash' >>$fname
echo ""	>>$fname

unset fname

#	End of setup.csh section.
#------------------------------------------------------------------------
#	Now set up the file $UNIDAQ/murmur_client/error/inc/murmur_env_alias
#	Do this only if an original install.

	if ( $nargs == 0 ) then
set fname=murmur_client/error/inc/murmur_env_alias
rm -f $fname

#	Put out the header portion first.

cat >$fname <<ZZZ
#
#	Choose the MURMUR_DIR standard needed on this machine.  Set the
#	server IP address appropriately as well.
#
ZZZ

#	Find out the top murmur directory.

echo "Setting the MURMUR root (optional; if not used, just take the defaults)."
echo -n "MURMUR_DIR = [/usr/murmur/v1_1] "
set ans=$<
if ( $ans == "" ) then
   set murroot=/usr/murmur/v1_1
else
   set murroot=$ans
endif

echo "setenv MURMUR_DIR         $murroot" >>$fname
unset murroot
echo ""

#	Find out the IP address where the murmur server is running.

echo "Setting the IP address where the murmur server will run."
echo "As with the MURMUR dir, if you don't use murmur, just take the default."
echo -n "MURMUR_SERVER_ADD = [134.3.30.43] "
set ans=$<
if ( $ans == "" ) then
   set addroot=134.3.30.43
else
   set addroot=$ans
endif

#	OK, this is the variable which was also to be written to the $savename
#	file.

echo "setenv MURMUR_SERVER_ADD  $addroot" >>$fname
echo "setenv MURMUR_SERVER_ADD  $addroot" >>$savename
unset addroot
echo ""

#	Write out the remaining lines of the file.

cat >>$fname <<ZZZ
setenv MURMUR_SYSDB       \$MURMUR_DIR/lib
setenv MURMUR_LOG         \$MURMUR_DIR/lib
setenv MURMUR_MESSACT     \$MURMUR_DIR/bin
#
# Allow access to MURMUR message utility and database dumping program
#

alias message       \${MURMUR_DIR}'/bin/message'
alias dumpdb        \${MURMUR_DIR}'/bin/dumpDb'
alias condb         \${MURMUR_DIR}'/bin/DbConcatenator'
alias mursendmsg    \${MURMUR_DIR}'/tests/mur_send_message'
alias mur_generrtab \${MURMUR_DIR}'/bin/mur_generrtab'
alias murIPCGen     \${MURMUR_DIR}'/bin/murIPCGen'
alias start_mursrvr \${MURMUR_DIR}'/bin/start_mursrvr'
alias stop_mursrvr  \${MURMUR_DIR}'/bin/stop_mursrvr'
alias start_murgui  \${MURMUR_DIR}'/bin/start_murgui'
alias murmur_snap   \${MURMUR_DIR}'/bin/murmur_snap'

#
# Set up the aliases for the UNIDAQ scripts which help in the
# database creation.
#
alias scratch_base  \${MURMUR_DIR}'/base_lib/scratch_base'
alias win3_base     \${MURMUR_DIR}'/base_display_lib/win3_base'
alias final_base    \${MURMUR_DIR}'/hold_db/final_base'
#
ZZZ

unset fname
	else
echo $hallm >>$savename
	endif

#	End of murmur_env_alias section.
#------------------------------------------------------------------------
#	Now set up the file src/control/logbook/logbook.h
#	Only do this if this is an original install.

	if ( $nargs == 0 ) then
set fname=src/control/logbook/logbook.h
rm -f $fname

#	Find out where to write the log book file.  For the original
#	installations, this will be $TOPUNIDAQ/log by default, as an
#	environmental variable, and also written to logbook.h

echo "Setting the location of the log book file."
echo 'Force log.bookYYMMDD files in $TOPUNIDAQ/log for system install. '
set logb='"$TOPUNIDAQ/log/log.book"'

echo "setenv LGBK_LOC           $logb"   >> $savename
echo "#define LOGBOOK  $logb " >  $fname
unset logb
echo ""

echo '#define MESSLIST "$(TOPUNIDAQ)/murmur_client/error/inc/mess.list"' >>$fname

unset fname
	endif

#------------------------------------------------------------------------
#	Make sure the specified C++ compiler is part of the user's PATH
#       No longer used as of v2.2, but save it for the future.

#  set findcc=`which $thiscc | awk ' { print substr($1,1,1) } ' `
#  if ( "$findcc" != "/" ) then
#    echo "WARNING!  The C++ compiler $thiscc is not part of your command PATH."
#    echo "          Please contact your system administrator before you make"
#    echo "          the UNIDAQ executables."
#  endif
#  unset findcc

#  unset thiscc

nodefile:
#------------------------------------------------------------------------
#	Skip to here from the VXWORKS camac Makefile section.
#	Write out the node_file

set fname=conf/nodefile
rm -f $fname

if ( $MACHINE == VXWORKS) then
   echo -n "Enter the VxWorks machine name (name.network format) [] "
   set tnode=$<
   set ttype=VXWORKS
else
   set tnode=`hostname`
   set ttype=UNIX
endif
echo $tnode $ttype >$fname
unset ttype

unset fname

#------------------------------------------------------------------------
#	Write out the bin/Start_hostname script

set fname=conf/Start_$tnode
rm -f $fname

	if ( $MACHINE == VXWORKS ) then
cat >$fname <<ZZZ
cd "$pdsave/conf"
ld <vxshm.o
cd "../NOVA/src"
ld <vxnovad
cd "../netsrc"
ld <vxnovand
cd "../.."
sp novad_main
sp novand_main
cd "bin"
ld <msg_server
sp msg_server
ld <status
ld <repair
ld <ask
ZZZ
ln -s $fname Start_$tnode
unset fname
unset tnode
unset pdsave

	else
cat >$fname <<ZZZ
!/bin/csh
onintr -
source \$TOPUNIDAQ/setup.csh
msg_server >& /dev/null &
xpc >& /dev/null  &
if ( ! \$?LOCAL_NOVA ) then
  setenv LOCAL_NOVA /tmp/nova
endif
if(-e \$LOCAL_NOVA) rm \$LOCAL_NOVA
novad \$LOCAL_NOVA  >& /dev/null  &
sleep 1
novand >& /dev/null &
ask xpc START YES runco
ask xpc START YES logbook
ask xpc START YES xpc_checker
ZZZ

chmod g+wx $fname
chmod u+x $fname
chmod o+x $fname

unset fname
unset tnode
	endif

#------------------------------------------------------------------------
#	Finish with the wind-up message.

cat <<ZZZ
                               ===========
Finished with the system-level installation.  Now you should put into 
either your .login or .cshrc file, the line 

	source $cwd/setup.csh

So you will always be set to use the software. The user-install should 
also be run. 

ZZZ

	if ( $nargs == 0 ) then
cat <<ZZZ
******* NOTE ******* NO BINARIES ARE A PART OF THIS DISTRIBUTION !!!!!

To complete the installation, source the above file now, and enter the 
following command:

	make

This will take anywhere from 5-10 minutes depending on the machine type
and create all libraries and executables needed by the default system.

-----------------------------------------------------------------------
ZZZ
	endif



