/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      August 28, 1992                                          */
/*        This is file cmdlib.c                                  */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef SGI 
#define _BSD_SIGNALS 
#endif 
#ifdef VXWORKS
#include <vxWorks.h>
/* #include <stdioLib.h> */
#include <msgQLib.h>
#include <wdLib.h>
#include <sigLib.h>
#include <ioLib.h>
#include <memLib.h>
#include <strLib.h>
#else
#include <signal.h>
#include <netdb.h>
#include <stdio.h>
#endif
#include "template.h"
#include "variable.h"
#include "status_path.h"

char Nova_Host[100] = "\0";
int  nova_prio = -99;          /*  default NOVA priority  */

int pid;

#ifdef VXWORKS
MSG_Q_ID msqid;
WDOG_ID wid;     
void wdtimeout();
#else
int msqid;
#endif

int interrupt_flag = 0;
char ownpname[15];
char this_node[100] = "\0";;
#ifdef SGI
int  cmd_signal_handler(); 
int  intr_signal_handler(), dummy(), Busy(), Set_Interrupt();
#else
void cmd_signal_handler(); 
void intr_signal_handler(), dummy(), Busy(), Set_Interrupt();
#endif
float stof();
void Init_hostname();
void Init_Status_Path();
int  Read_from_MsgQ();
struct command_list *first_cmd = NULL, *last_cmd = NULL;


/*(a+out) ------------------------------------------------- */
void Template_Main(Process_Event, Process_Command_Composite, Process_Idle)
int (*Process_Event)();
void (*Process_Command_Composite)();
void (*Process_Idle)();
/*(b+out) -------------------------------------------------  
Procedure Name
Template_Main - main procedure of the template
Synopsis
Description
Template_Main(process_name, Process_Event, Process_Idle)
contains the main loop of the template, which waits for command 
messages or event data. It gets the addresses of the following
four procedures from the user program: 
Process_Event: is the procedure which processes the event data. 
Process_Command_Composite : is the procedure which handles the
user defined composite commands (level 3 commands).
Process_Idle : is the procedure to be called periodically when 
there are command messages and event data.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
  char tmp[200],full[200];
  int flag = TRUE;

/*** Also remove file $USERUNIDAQ/log/var/ownpname ***/
  sprintf(tmp,"$(USERUNIDAQ)/log/var/%s",ownpname);
  if(decode_filename(tmp,full) == 0) unlink(full);
  /*
   *   Wait for a command or an event
   */
  while (flag == TRUE) {
#ifdef VXWORKS
    wdStart(wid,sysClkRateGet(),(FUNCPTR)wdtimeout,pid);
#else
/*    alarm(1); */
#endif
    interrupt_flag = 0;
    check_event(Process_Event);
    flag = check_command(Process_Command_Composite);
    if ((first_cmd == NULL) && (flag == TRUE)) {
      if (Process_Idle != NULL) (*Process_Idle)();
      else idle(1);
    }
  }
#ifdef VXWORKS
  wdDelete(wid);
#endif
}


int Template_Init(process_name)
char *process_name;
{
   int index;
   Table entry;
#ifdef VXWORKS
   SIGVEC vec;
#else
   struct sigvec vec;
#endif
   char nlist[50][100];
   int n_nodes,i;
   int result;

   rpc_init();
#ifdef VXWORKS
   wid = wdCreate();
#endif
   Init_Table_and_MsgQ(process_name);
   Init_hostname();
   if (strlen(Nova_Host) != 0)  {
     if (strncmp(this_node, Nova_Host, strlen(Nova_Host)) == 0) 
                            *Nova_Host = '\0';
   }

#if defined(SGI)
   vec.sv_handler = (int (*)()) dummy;
#else 
#if defined(VXWORKS)
   vec.sv_handler = (VOIDFUNCPTR) dummy;
#else
   vec.sv_handler = (void (*)()) dummy;
#endif
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGALRM, &vec, (struct sigvec *) NULL);
#else 
#if defined(VXWORKS)
   sigvec(SIGALRM, &vec, (SIGVEC *) NULL);
#else
   sigvec(SIGALRM, &vec, (struct sigvec *) NULL);
#endif
#endif
#ifdef SGI
   vec.sv_handler = (int (*)()) Busy;
#else 
#if defined(VXWORKS)
   vec.sv_handler = (VOIDFUNCPTR) Busy;
#else
   vec.sv_handler = (void (*)()) Busy;
#endif
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(BUSY_SIGNAL, &vec, (struct sigvec *) NULL);
#else 
#if defined(VXWORKS)
   sigvec(BUSY_SIGNAL, &vec, (SIGVEC *) NULL);
#else
   sigvec(BUSY_SIGNAL, &vec, (struct sigvec *) NULL);
#endif
#endif
#ifdef SGI
   vec.sv_handler = (int (*)()) Set_Interrupt;
#else 
#if defined(VXWORKS)
   vec.sv_handler = (VOIDFUNCPTR) Set_Interrupt;
#else
   vec.sv_handler = (void (*)()) Set_Interrupt;
#endif
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGTERM, &vec, (struct sigvec *) NULL);
#else 
#if defined(VXWORKS)
   sigvec(SIGTERM, &vec, (SIGVEC *) NULL);
#else
   sigvec(SIGTERM, &vec, (struct sigvec *) NULL);
#endif
#endif

   /* 
    *  Before adding the entry make sure it is not already there and alive
    */
   index = Search_Table(ownpname);
   if (index >= 0) {
      Read_from_Table(&entry,index);
      if (entry.status != DEAD) {
	if((strncmp(entry.node,this_node,strlen(this_node)) == 0 ) ||
	   (entry.node[0] == '\0'))
	  return(-1);
      }
   }
  /*
   * Also on the remote NODES
   */
   rpc_get_nodes(nlist,100,&n_nodes);
   i = 0;
   result = -1;
   while((i<n_nodes) &&(result != 1)){
     if(strncmp(this_node,nlist[i],strlen(nlist[i])) != 0)
       result = rpc_Check_Process(nlist[i],ownpname);
     i++;
   }
   if(result == 1) {
     rpc_Update_Table(index,ownpname,nlist[i-1],ALIVE);
     return(-2);
   }
   rpc_Update_Table(index,ownpname,entry.node,DEAD);
   Init_Status_Path();
   Add_Entry_to_Table();
   return(0);
}

void Init_hostname()
{
  char *t,*n;
#if !defined(VXWORKS)
  struct hostent *h;
#endif
  int i = 0;
  char nlist[50][100];
  int n_nodes;

/*** First get the easy hostname ***/
  gethostname(this_node,50);
#if !defined(VXWORKS)
/*** Use the official name to see if it can be improved **/
  h = gethostbyname(this_node);
  n = h->h_name;
  if(strlen(n) < strlen(this_node)) n = this_node;
/*** Pick the longest alias ***/
  t = h->h_aliases[i++];
  while(t != 0){
    if((strncmp(t,n,strlen(n)) == 0) &&
       (strlen(t) > strlen(n)) &&
       (strchr(t,'.') != NULL)) n = t;
    t = h->h_aliases[i++];
  }
  strcpy(this_node,n);
#endif
/*** Use the nodefile ***/
  rpc_get_nodes(nlist,100,&n_nodes);
  for(i=0;i<n_nodes;i++){
    if(strncmp(this_node,nlist[i],strlen(this_node)) == 0) {
      if((strlen(nlist[i]) > strlen(this_node)) &&
         (strchr(nlist[i],'.') != NULL) ) strcpy(this_node,nlist[i]);
    }
  }
}



char *get_name_from_args(argc,argv)
int argc;
char **argv;
{
  int i, j;
  char *p_name;

  for (i=0; ( (i<argc-1) && ( (j=strcmp(argv[i],"-m")) != 0) ); i++);
  if ( (j==0) && (i<(argc-1)) )  strcpy(Nova_Host,argv[i+1]);

  for (i=0; ( (i<argc-1) && ( (j=strcmp(argv[i],"-p")) != 0) ); i++);
  if ( (j==0) && (i<(argc-1)) )  {
    sscanf(argv[i+1],"%d",&nova_prio);
    if (nova_prio < 0)   nova_prio=1;
    if (nova_prio > 100) nova_prio=100;
  }

  i = 0;
  while((i < argc-1) && (strcmp(argv[i],"-n") != 0))i++; 
  if((strcmp(argv[i],"-n") == 0) && ( i<(argc-1))) return(argv[i+1]);
  else{        
    if((p_name = (char *)strrchr(argv[0],'/')) == NULL) p_name = argv[0];
    else                                        p_name++;
    return(p_name);
  }
}


/*(a+out) ------------------------------------------------- */
int check_event(Process_Event)
int (*Process_Event)();
/*(b+out) ------------------------------------------------- 
Procedure Name
check_event - 
Synopsis
Description
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   if (Process_Event != NULL) return((*Process_Event)());
   else return(FALSE);
}


is_there_command()
{
   char string[CMD_LENGTH];
   struct command_list *p;

   while (Read_from_MsgQ(msqid, string) >= 0) { 
      p = (struct command_list *) malloc(sizeof(struct command_list));
      strcpy(p->string, string);
      p->next = NULL;
      if (first_cmd == NULL) {
         first_cmd = last_cmd = p;
      }
      else {
         last_cmd->next = p;
         last_cmd = p;
      }
   }
}


/*(a+out) ------------------------------------------------- */
int check_command(Process_Command_Composite)
void (*Process_Command_Composite)();
/*(b+out) ------------------------------------------------- 
Procedure Name
check_command - checks whether there are commands in the command list 
Synopsis
Description
check_command(Process_Command_Composite) checks the command list to
see whether there are incoming command messages or not. If there is
no waiting command messages, it returns TRUE. Otherwise, it calls
the Process_Command() procedure. 
Process_Command_Composite is the procedure which handles the user
defined composite (level 3) commands.
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char string[CMD_LENGTH];
   struct command_list *p;
    
   is_there_command();
   if (first_cmd == NULL) return(TRUE);
   else {
      p = first_cmd;
      first_cmd = first_cmd->next; 
      if (first_cmd == NULL) last_cmd = NULL;
      strcpy(string, p->string);
      free((char *)p);
      return(Process_Command(string, Process_Command_Composite));
   }
}




/*(a+out) ------------------------------------------------- */
int Process_Command(string, Process_Command_Composite)
char *string;
void (*Process_Command_Composite)();
/*(b+out) ------------------------------------------------- 
Procedure Name
Process_Command - calls the corresponding command procedure 

Synopsis
Description
Process_Command(string, Process_Command_Composite) parses the string, 
replaces the tokens, and calls the corresponding command procedure.
The command might belong to either system level, process level, or 
user level (composite).  
string: is the command message which contains the sender process name
and the command string.   
Process_Command_Composite: is the procedure which handles the user
defined composite commands (level 3 commands). 

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   int cmd_code_sys, cmd_code_usr;
   char arg_string[CMD_LENGTH], header[CMD_LENGTH];

   if (debug_level >= 20) printf("%s\n", string);
   Replace_Tokens(string);

      /*
       *   Search the User Command Dictionary for a full-length match
       */
   if ( (cmd_code_usr = 
              Parse_Command_Usr_Exact(string, header, arg_string) ) >= 0) {
      Parse_for_Set ( arg_string );
      return (*(cmd_dict_usr[cmd_code_usr].proc_addr))(header, arg_string);
   }
      /*
       *   Search the System Command Dictionary for a full-length match
       */
   else if ( (cmd_code_sys = 
              Parse_Command_Sys_Exact(string, header, arg_string) ) >= 0) {
      return (*(cmd_dict_sys[cmd_code_sys].proc_addr))(header, arg_string);
   }
      /*
       *   Search the User Command Dictionary for a partial match
       */
   else if ( (cmd_code_usr = 
              Parse_Command_Usr_Part(string, header, arg_string) ) >= 0) {
      Parse_for_Set ( arg_string );
      return (*(cmd_dict_usr[cmd_code_usr].proc_addr))(header, arg_string);
   }
   else if (cmd_code_usr == AMBIGUOUS_CMD) {
      printf("Ambiguous User command\n");
      return TRUE;
   }
   else if (cmd_code_usr == NOT_CMD) {
      /*
       *   Search the System Command Dictionary for a partial match
       */
      cmd_code_sys = Parse_Command_Sys_Part(string, header, arg_string);
      if (cmd_code_sys >= 0) {
         return (*(cmd_dict_sys[cmd_code_sys].proc_addr))(header, arg_string);
      }
      else if (cmd_code_sys == AMBIGUOUS_CMD) {
         printf("Ambiguous System command\n");
         return TRUE;
      }
      else if (cmd_code_sys == NOT_CMD) {
         /*
          *   Search the Composite Commands
          */
         if (Process_Command_Composite != NULL)
            (*Process_Command_Composite)(string);
         return TRUE;
      }
   }
}






/*(a+out) ------------------------------------------------- */
int Ackn(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Ackn - system level ackn command 

Synopsis
Description
Ackn(header, arg_string) processes an ackn command.          
header: the name of the process which sends the ackn command,
arg_string: the arguments of the ackn. 
				
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char msg[CMD_LENGTH], temp[CMD_LENGTH];
 
   /*
    *   Execute the command.
    */
   strcpy(msg, header);
   strcat(msg, " ");
   strcat(msg, arg_string);
   Process_Command(msg, NULL);

   /*
    *   Reply the sender process.
    */
   strcpy(temp, "REPLY");
   strcat(temp, " Done");
   Send_Message(header, temp);
 
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Ask(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Ask - system level ask command 
Synopsis
Description
Ask(header, arg_string) processes an ask command.
header: the name of the process which sends the ask command, 
arg_string: the arguments of the ask command.
		   
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char process_name[PROCESS_NAME], verb[CMD_LENGTH];
   char temp[CMD_LENGTH];

   Parse(arg_string, process_name, verb);

   /* 
    *   Send ackn to the corresponding process. 
    */
   strcpy(temp, "ACKN");
   strcat(temp, verb);
   Send_Message(process_name, temp);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Exit(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Exit - system level exit handler 
Synopsis
Description
Exit(header, arg_string) updates the status of the process  
in CT from ALIVE to DEAD, and detaches the MsgQ and CT.     
header: the name of the process which sends the exit command,
arg_string: the arguments of the exit command.
			  
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */ 

{
   int index;
   Table entry;

   /*
    *   Modify the Common Table
    */
   index = Search_Table(ownpname);
   Read_from_Table(&entry, index);
   entry.status = DEAD;
   Write_to_Table(&entry, index);

   /*
    *   Delete the Msgq and the Table
    */
   if (Delete_MsgQ(entry.msgq_id) == ERROR)
      if (debug_level >= 10) printf("Error in Remove_MsgQ()\n");

   if (Detach_Table() == ERROR)
      if (debug_level >= 10) printf("Error in Detach_Table...\n");

/*** remove process from the status path ***/
   remove_proc_from_sp(ownpname);
   unlink_sp();

   return FALSE;
}




/*(a+out) ------------------------------------------------- */
int GetPar(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
GetPar - system level get parameter command 
Synopsis
Description

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   int  *int_addr, index;
   char *char_addr, var_name[CMD_LENGTH];
   char msg[CMD_LENGTH], temp1[CMD_LENGTH], temp2[CMD_LENGTH];

   if (debug_level >= 20) printf("Get_par() = %s\n", arg_string);
   if (strncmp(header, "user",4) == 0) strcpy(header, ownpname);
   strcpy(msg, "SetPar ");

   Parse(arg_string, var_name, temp1);
   while (strcmp(var_name, "\0") != 0) {
      GetVariable(var_name, temp2);
      strcat(msg, temp2);
      Parse(temp1, var_name, temp2);
      strcpy(temp1, temp2);
   }
   
   Send_Message(header, msg);
   return TRUE;  
}




/*(a+out) ------------------------------------------------- */
int Mail(header, arg_string)
char *header, *arg_string;

/*(b+out) ------------------------------------------------- 
Procedure Name
Mail - system level mail command 
Synopsis
Description
Mail(header, arg_string) processes a mail command.           
header: the name of the process which sends the mail command,
arg_string: the arguments of the end command. 
				
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char string[100];

   strcpy(string, "Mail Received: ");
   strcat(string, arg_string);
   printf("%s\n", string);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Reply(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Reply - system level reply command 
Synopsis
Description
Reply(header, arg_string) processes a reply command.         
header: the name of the process which sends the reply cmnd,   
arg_string: the arguments of the reply command.
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */ 

{
   if (debug_level > 10) printf("Reply Received: %s", arg_string);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Set(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Set - system level set command 
Synopsis
Description
Set(header, arg_string) assigns a new value to a variable.   
header: the name of the process which sends the set command,
arg_string: the arguments of the set command.   
			
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char temp[CMD_LENGTH];

   SetVariable(arg_string, temp);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int SetPar(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
SetPar - system level set parameter command 
Synopsis
Description

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char temp1[CMD_LENGTH], temp2[CMD_LENGTH];

   if (debug_level >= 20) printf("Set_par() = %s\n", arg_string);

   strcpy(temp1, arg_string);
   while (sscanf(temp1,"%s",temp2) > 0) {
      SetVariable(temp1, temp2);
      strcpy(temp1, temp2);
   }

   return TRUE;
}




/*(a+out) ------------------------------------------------- */
int Shell(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Shell - system level shell command
Synopsis
Description
Shell(header, arg_string) processes a UNIX shell command.    
header: the name of the process which sends the shell cmnd,  
arg_string: the arguments of the shell command. 
			           
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
#ifdef VXWORKS
  printf("This command is not supported in VxWorks\n");
#else
  system(arg_string);
#endif
  return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Show(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Show - system level show command  
Synopsis
Description
Show(header, arg_string) displays the value of a variable.   
header: the name of the process which sends the show command,
arg_string: the arguments of the show command.     
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char  msg[CMD_LENGTH], temp[CMD_LENGTH];
   char proc_n[100],host_n[100];

   Get_Proc_and_Host(header,proc_n,host_n);
   if (strcmp(proc_n, "user") == 0) strcpy(header, ownpname);
   strcpy(msg, "mail show result: ");
   GetVariable(arg_string, temp);
   strcat(msg, temp);

   /*
    *   Sends the value to the sender process.
    */
   Send_Message(header, msg);
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int Tell(header, arg_string)
char *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Tell -  system level tell command
Synopsis
Description
Tell(header, arg_string) processes a tell command.           
header: the name of the process which sends the tell command,
arg_string: the arguments of the tell command.           
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char process_name[PROCESS_NAME], verb[CMD_LENGTH];

   Parse(arg_string, process_name, verb);
   Send_Message(process_name, verb);
   return TRUE;
}


/*(a+out) ------------------------------------------------- */
int Parse_Command_Sys_Part(string, header, arg_string)
char *string, *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Command_Sys_Part - returns a command_code 
Synopsis
Description
Parse_Command_Sys_Part(string, header, arg_string) parses a given 
string based on cmd_dict_sys and returns the command_code.  A sufficient
match occurs if the given command matches the leading characters of the full
command.  
string: the string that will be parsed,                      
header: the name of the sender process,                      
arg_string : the arguments of the command.  
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char command[CMD_LENGTH], temp[CMD_LENGTH];
   int cmd_code = NOT_CMD;
   int i;

   Parse(string, header, temp);
   Parse(temp, command, arg_string); 
   for (i = 0; i < cmd_number_sys; i++) {
      if (strncasecmp(command, cmd_dict_sys[i].name, strlen(command)) == 0) { 
         if (debug_level >= 10) printf("%d. %s\n", i, cmd_dict_sys[i].name);
         if (cmd_code == NOT_CMD) cmd_code = i;
         else cmd_code = AMBIGUOUS_CMD;
      }
   } 
   return cmd_code;
}

/*(a+out) ------------------------------------------------- */
int Parse_Command_Sys_Exact(string, header, arg_string)
char *string, *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Command_Sys_Exact - returns a command_code 
Synopsis
Description
Parse_Command_Sys_Exact(string, header, arg_string) parses a given 
string based on cmd_dict_sys and returns the command_code if the match is
exact.  
string: the string that will be parsed,                      
header: the name of the sender process,                      
arg_string : the arguments of the command.  
				 
Superconducting Super Collider Laboratory 1.2
(c+out) ------------------------------------------------- */

{
   char command[CMD_LENGTH], temp[CMD_LENGTH];
   int cmd_code = NOT_CMD;
   int i;

   Parse(string, header, temp);
   Parse(temp, command, arg_string); 
   for (i = 0; i < cmd_number_sys; i++) {
      if (strcasecmp(command, cmd_dict_sys[i].name) == 0) {
         if (debug_level >= 10) printf("%d. %s\n", i, cmd_dict_sys[i].name);
         cmd_code = i;
         return cmd_code;
      }
   } 
   return cmd_code;
}


/*(a+out) ------------------------------------------------- */
int Parse_Command_Usr_Part(string, header, arg_string)
char *string, *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Command_Usr_Part - returns the command_code
 
Synopsis
Description
Parse_Command_Usr_Part(string, header, arg_string) parses a given 
string based on cmd_dict_usr and returns the command_code.  A sufficient
match occurs if the given command matches the leading characters of the full
command.  
string: the string that will be parsed,                      
header: the name of the sender process,                      
arg_string : the arguments of the command.  
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char command[CMD_LENGTH], temp[CMD_LENGTH];
   int cmd_code = NOT_CMD;
   int i;

   Parse(string, header, temp);
   Parse(temp, command, arg_string); 
   for (i = 0; i < cmd_number_usr; i++) {
      if (strncasecmp(command, cmd_dict_usr[i].name, strlen(command)) == 0) { 
         if (debug_level >= 10) printf("%d. %s\n", i, cmd_dict_usr[i].name);
         if (cmd_code == NOT_CMD) cmd_code = i;
         else cmd_code = AMBIGUOUS_CMD;
      }
   } 
   return cmd_code;
}


/*(a+out) ------------------------------------------------- */
int Parse_Command_Usr_Exact(string, header, arg_string)
char *string, *header, *arg_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Command_Usr_Exact - returns the command_code
 
Synopsis
Description
Parse_Command_Usr_Exact(string, header, arg_string) parses a given 
string based on cmd_dict_usr and returns the command_code if the match is
exact.
string: the string that will be parsed,                      
header: the name of the sender process,                      
arg_string : the arguments of the command.  
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char command[CMD_LENGTH], temp[CMD_LENGTH];
   int cmd_code = NOT_CMD;
   int i;

   Parse(string, header, temp);
   Parse(temp, command, arg_string); 
   for (i = 0; i < cmd_number_usr; i++) {
      if (strcasecmp(command, cmd_dict_usr[i].name) == 0) {
         if (debug_level >= 10) printf("%d. %s\n", i, cmd_dict_usr[i].name);
         cmd_code = i;
         return cmd_code;
      }
   } 
   return cmd_code;
}


/*(a+out) ------------------------------------------------- */
Parse_for_Set ( arg_string )
char *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Parse_for_Set - pick out variables which can be set from arg_string
Synopsis
Description
Parse_for_Set(arg_string) picks out the name of variables and the
values to which they should be set from arg_string, sets them, and
returns arg_string with these variables/values removed.
arg_string: the string to be parsed.

(c+out) -------------------------------------------------- */

{
  char temp1[MAX_MSG_SIZE], temp2[MAX_MSG_SIZE], temp3[MAX_MSG_SIZE];
  char endcom[MAX_MSG_SIZE];
  int j;

  endcom[0] = '\0';
  if (debug_level >= 20) printf ("Parse_for_Set: in arg=%s\n", arg_string);
  Parse (arg_string, temp1, temp2);
  while (strcmp(temp1, "\0") != 0)  {
    if ( Parse_Var_Usr(temp1,temp3) > 0 )  {
      strcpy (temp3, temp1);
      strcat (temp3, temp2);
      SetVariable(temp3, temp2);
    }
    else if ( Parse_Var_Sys(temp1, temp3) > 0 ) {
      strcpy (temp3, temp1);
      strcat (temp3, temp2);
      SetVariable(temp3, temp2);
    }
    else  {
      strcat (endcom, " ");
      strcat (endcom, temp1);
    }
    strcpy (temp3, temp2);
    Parse (temp3, temp1, temp2);
  }
  strcpy (arg_string, endcom);
  if (debug_level >= 20) printf ("Parse_for_Set: out arg=%s\n", arg_string);
}


/*(a+out) ------------------------------------------------- */
int Parse_Var_Sys(string, rest)
char *string, *rest;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Var_Sys - returns the index of the variable
Synopsis
Description
Parse_Var_Sys(string, rest) parses a given string based on  
var_dict_sys and returns the index of the variable.          
string: the string that will be parsed,                     
rest  : the rest of the string after the first word removed. 
			
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char var_name[CMD_LENGTH];
   int  flag, i;

   Parse(string, var_name, rest);
   flag = 0; i = 0;
   while ((flag == 0) & (i < var_number_sys)) {
      if (strcasecmp(var_name, var_dict_sys[i].name) == 0) flag = 1;
      else i++;
   }
   if (flag == 0) return -1;
   else return i;
}





/*(a+out) ------------------------------------------------- */
int Parse_Var_Usr(string, rest)
char *string, *rest;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse_Var_Usr - returns the index of the variable
Synopsis
Description
Parse_Var_Usr(string, rest) parses a given string based on   
var_dict_usr and returns the index of the variable.          
string: the string that will be parsed,                      
rest  : the rest of the string after the first word removed. 
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char var_name[CMD_LENGTH];
   int  flag, i;

   Parse(string, var_name, rest);
   flag = 0; i = 0;
   while ((flag == 0) & (i < var_number_usr)) {
      if (strcasecmp(var_name, var_dict_usr[i].name) == 0) flag = 1;
      else i++;
   }
   if (flag == 0) return -1;
   else return i;
}





/*(a+out) ------------------------------------------------- */
Parse(string, first, rest)
char *string, *first, *rest;
/*(b+out) ------------------------------------------------- 
Procedure Name
Parse - parses a given string 
Synopsis
Description
Parse(string, first, rest) parses a given string.            
string: the string that will be parsed,                      
first : the first word in the string,                        
rest  : the rest of the string after the first word removed. 
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char temp[CMD_LENGTH];
   int  i, j;

   i = 0;
   while (((string[i]==' ') | (string[i]=='\t') | (string[i]=='\0')) & (i < strlen(string)))
      i++;
   j = i;
   if (string[j] == '=') j++;
   else while (((string[j] != ' ') & (string[j] != '\t') & 
                (string[j] != '=')) & (j < strlen(string))) j++;
   strncpy(first, string+i, j-i);
   first[j-i] = '\0';
   strcpy(rest, string+j); 
}

Get_Proc_and_Host(full,dest,node)
char *full,*dest,*node;
{
  char *tmp;

  node[0] = '\0';
  strcpy(dest,full);
  if((tmp = (char *)strrchr(dest,'@')) != NULL){
    *tmp = '\0';
    tmp++;
    strcpy(node,tmp);
  }
}

/*(a+out) ------------------------------------------------- */
int Send_Message(process_name, message)
char *process_name, *message;
/*(b+out) ------------------------------------------------- 
Procedure Name
Send_Message - writes a message to a message queue
Synopsis
Description
Send_Message(process_name, message) writes a message to a    
process' message queue, and sends a signal to that process.
process_name: name of the process the message will be sent.  
message: the message string.                              
			 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char msg[1000];

   char node[100];
   char mssend[1000],dest[100],snd_dest[100];
   int index;
   Table entry;

   char nlist[50][100];
   int i,n_nodes;
   int result;

   Get_Proc_and_Host(process_name,dest,node);
   if(strlen(this_node) == 0) {
     Init_hostname();
     rpc_init();
   }
   Add_Header(message, msg);
   index = Search_Table(dest);

   if(node[0] != '\0') {
     if((strncmp(node,this_node,strlen(this_node)) == 0) ||
        (strncmp(node,this_node,strlen(node)) == 0))
       return(send_to_single(dest,msg));
     else {
       result = rpc_Send_Message(process_name,msg);
       if(result >= 0) rpc_Update_Table(index,dest,node,ALIVE);
       else            rpc_Update_Table(index,dest,node,DEAD);
       return(result);
     }
   } 

    if(index >= 0) {
      Read_from_Table(&entry,index);
      if((strncmp(entry.node,this_node,strlen(entry.node)) == 0) ||
	 (entry.node[0] == '\0')) {
	if(entry.status != DEAD) return(send_to_single(dest,msg));
	else result = 0;
      }
      else if(entry.status != DEAD) {
	sprintf(snd_dest,"%s%c%s",dest,'@',entry.node);
        result = rpc_Send_Message(snd_dest,msg);
	if(result < 0) rpc_Update_Table(index,dest,entry.node,DEAD);
	else           return(result);
      }
    }

    if((result = rpc_get_nodes(nlist,100,&n_nodes)) < 0) return(result);
    result = -1;
    i = 0;
    while((i<n_nodes) && (result != 1)){
      if(strncmp(this_node,nlist[i],strlen(nlist[i])) != 0)
        result = rpc_Check_Process(nlist[i],dest);
      i++;
    }
    if(result == 1) {
/** Add entry to common table **/
      rpc_Update_Table(index,dest,nlist[i-1],ALIVE);
/** Send Message **/
      sprintf(snd_dest,"%s%c%s",dest,'@',nlist[i-1]);
      return(rpc_Send_Message(snd_dest,msg));
    }
/** Update table if needed **/
    rpc_Update_Table(index,dest,nlist[i-1],DEAD);
    return(-12);
}


int send_to_single(dest,message)
char *dest;
char *message;
/* ////////////////////////////////////////////////////////////////////
// Note: The message should start with the name of the sending process.
// This is the responsibility of the programmer who uses this routine.
//////////////////////////////////////////////////////////////////// */
{
	int index;
	Table entry;

/* // destination is any process */
	index = Search_Table(dest);
	if(index < 0) {
#ifdef _DEBUG
	  printf("Sending to process %s failed; process is not known\n",dest);
#endif
	  return(-3);
	}
	Read_from_Table(&entry,index);
        if((strncmp(entry.node,this_node,strlen(entry.node)) == 0) ||
	  (entry.node[0] == '\0')) {
	  if(entry.status == DEAD) {
#ifdef _DEBUG
	    printf("Sending to process %s failed; process is dead\n",dest);
#endif
	    return(-1);
	  }
	  Write_to_MsgQ(entry.msgq_id,message);
#ifdef _DEBUG
	  printf("Signalling process %d\n",entry.process_id);
#endif
	  kill(entry.process_id,MSG_SIGNAL);
	  return(0);
	}
	return(-2);
}


/*(a+out) ------------------------------------------------- */
Add_Header(raw_msg, message)
char *raw_msg, *message;
/*(b+out) ------------------------------------------------- 
Procedure Name
Add_Header -  adds the name of the process to the message
 
Synopsis
Description
Add_Header(raw_msg, message) adds the name of the process to 
the beginning of the message.                                
raw_msg: initial message,                                   
message: the message with the header.   
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */ 

{
   message[0] = '\0';
   if(strcmp(ownpname,"") != 0){
     if (this_node[0] == '\0') Init_hostname();
     strcpy(message, ownpname);
     strcat(message,"@"); 
     strcat(message,this_node);
     strcat(message, " ");
   }
   strcat(message, raw_msg);
}





/*(a+out) ------------------------------------------------- */
Init_Table_and_MsgQ(loc_process_name)
char *loc_process_name;
/*(b+out) ------------------------------------------------- 
Procedure Name
Init_Table_and_MsgQ - initializes the Common Table
 
Synopsis
Description
Init_Table_and_MsgQ() initializes the Common Table and the   
MsgQ; connects the EVENT_SIGNAL and the MSG_SIGNAL to the    
signal handlers, and creates the semaphores.         
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
#ifdef VXWORKS
  SIGVEC vec;
#else
  struct sigvec vec;
#endif

   strcpy(ownpname, loc_process_name);
   strcpy(process_name, loc_process_name);

   if (Attach_Table(ownpname) == ERROR)
      printf("Error in Attach_Table()\n");

   if (Create_MsgQ(&msqid) == ERROR)
      printf("Error in Create_MsgQ()\n");

   /* 
    *   Set signal handler for event and command signals.
    */
#ifdef SGI
   vec.sv_handler = (int (*)()) cmd_signal_handler;
#else 
#if defined(VXWORKS)
   vec.sv_handler = (VOIDFUNCPTR) cmd_signal_handler;
#else 
   vec.sv_handler = (void (*)()) cmd_signal_handler;
#endif
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(MSG_SIGNAL, &vec, (struct sigvec *) NULL);
#else 
#if defined(VXWORKS)
   sigvec(MSG_SIGNAL, &vec, (SIGVEC *) NULL);
#else
   sigvec(MSG_SIGNAL, &vec, (struct sigvec *) NULL);
#endif
#endif
#ifdef SGI
   vec.sv_handler = (int (*)()) intr_signal_handler;
#else 
#if defined(VXWORKS)
   vec.sv_handler = (VOIDFUNCPTR) intr_signal_handler;
#else
   vec.sv_handler = (void (*)()) intr_signal_handler;
#endif
#endif
   vec.sv_mask = 0;
   vec.sv_flags = 0;
#ifdef HP
   sigvector(SIGINT, &vec, (struct sigvec *) NULL); 
#else 
#if defined(VXWORKS)
   sigvec(SIGINT, &vec, (SIGVEC *) NULL);
#else
   sigvec(SIGINT, &vec, (struct sigvec *) NULL); 
#endif
#endif
   if (debug_level >= 10) printf("CT and MsgQ were created...\n");
   if (debug_level >= 10) printf("msqid = %d\n", msqid);
}

void Init_Status_Path()
{
  int i;
  int st,size = 0;
  char p_name[80];

/*** link to the status path ***/
  if(link_to_sp() < 0) return;
/*** calculate the amount of space needed, note: SP_MAXCHAR characters
     will be used for STRING-type variables ***/
  for(i=0;i<var_number_sys;i++){
    if(var_dict_sys[i].type == INTEGER) st = sizeof(int);
    else if(var_dict_sys[i].type == REAL) st = sizeof(float);
    else st = SP_MAXCHAR;
/*** do not forget some overhead of the status path (in the 7 bytes) ***/
    size+=(7+st+strlen(var_dict_sys[i].name));
  }
  for(i=0;i<var_number_usr;i++){
    if(var_dict_usr[i].type == INTEGER) st = sizeof(int);
    else if(var_dict_usr[i].type == REAL) st = sizeof(float);
    else st = SP_MAXCHAR;
/*** do not forget some overhead of the status path (in the 7 bytes) ***/
    size+=(7+st+strlen(var_dict_usr[i].name));
  }
/*** add the process to the path ***/
  if(add_proc_to_sp(size) < 0) {
    strcpy(p_name,ownpname);
    ownpname[0]='\0';
    remove_proc_from_sp(p_name);
    strcpy(ownpname,p_name);
    if(add_proc_to_sp(size) < 0) {
      return;
    }
  }
/*** add variables to the status path ***/
  for(i=0;i<var_number_usr;i++){
    if(var_dict_usr[i].type == INTEGER) size = sizeof(int);
    else if(var_dict_usr[i].type == REAL) size = sizeof(float);
    else                                  size = SP_MAXCHAR;
    add_variable_to_sp(var_dict_usr[i].name,var_dict_usr[i].type,size,
                       (char *)(var_dict_usr[i].addr));
  }
  for(i=0;i<var_number_sys;i++){
    if(var_dict_sys[i].type == INTEGER) size = sizeof(int);
    else if(var_dict_sys[i].type == REAL) size = sizeof(float);
    else                                  size = SP_MAXCHAR;
    add_variable_to_sp(var_dict_sys[i].name,var_dict_sys[i].type,size,
                       (char *)(var_dict_sys[i].addr));
  }
}



/*(a+out) ------------------------------------------------- */
Add_Entry_to_Table()
/*(b+out) ------------------------------------------------- 
Procedure Name
Add_Entry_to_Table - inserts an entry into the CT
 
Synopsis
Description
Add_Entry_to_Table() inserts an entry into the CT, or updates
the status of an existing entry.                             
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   int index;
   Table entry;
 
   index = Search_Table(ownpname);
   if (debug_level >= 20) printf("In Add_Entry_to_Table: index = %d\n", index);
   if (index == -1) {   /* There is no entry for this process */
      strcpy(entry.process_name, ownpname);
      entry.node[0] = '\0';
#ifdef VXWORKS
      pid = entry.process_id = taskIdSelf();
#else
      pid = entry.process_id = getpid();
#endif
      entry.msgq_id = msqid;
      entry.status = ALIVE;
      if (debug_level >= 20) printf("entry.process_id = %d\n", entry.process_id);
      Add_to_Table(&entry);
   }
   else {   /* There is an entry; check the status */
      Read_from_Table(&entry, index);
      if (entry.status == DEAD) {
         entry.node[0] = '\0';
#ifdef VXWORKS
         pid = entry.process_id = taskIdSelf();
#else
         pid = entry.process_id = getpid();
#endif
         entry.msgq_id = msqid;
         entry.status = ALIVE;
         Write_to_Table(&entry, index);
         if (debug_level >= 20) printf("Status Changed!!!\n");
      }  
   }
 
   if (debug_level >= 10) printf("Common Table is updated...\n");
}





/*(a+out) ------------------------------------------------- */
Replace_Tokens(string)
char *string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Replace_Tokens - replaces all the tokens on a string 
 
Synopsis
Description
Replace_Tokens(string) replaces all the tokens on a string.  
string: the string for which the tokens will be replaced.    

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char command[CMD_LENGTH], first[CMD_LENGTH], rest[CMD_LENGTH];
   char replaced[CMD_LENGTH];

   strcpy(command, string);
   strcpy(replaced, "");
   Parse(command, first, rest); 
   while (strlen(first) > 0) {
      if (first[0] == '&') Token_Replace(first);
      strcat(replaced, first);
      strcat(replaced, " ");
      strcpy(command, rest);
      Parse(command, first, rest); 
   }
   strcpy(string, replaced); 
}





/*(a+out) ------------------------------------------------- */
Token_Replace(string)
char *string;
/*(b+out) ------------------------------------------------- 
Procedure Name
Token_Replace -  replaces a given token with its value 
 
Synopsis
Description
Token_Replace(string) replaces a given token with its value. 
string: the token to be replaced.                           

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   char token[CMD_LENGTH], temp[CMD_LENGTH], *ptemp;
   int index, *ptr;
   float *flt;

   strcpy(token, string+1);
   if (token[0] == '&') strcpy(string, token);
   else {
      index = Parse_Var_Usr(token, temp);
      if (index != -1) { 
         /*
          *   Replace the token with var_dict_usr. 
          */
         if (var_dict_usr[index].type == STRING) {
            ptemp = (char *)(var_dict_usr[index].addr);
            strcpy(string, ptemp);
         }
         else if (var_dict_usr[index].type == INTEGER) { 
            ptr = (int *)(var_dict_usr[index].addr);
            sprintf(string, "%d", *ptr);
         }
          else if (var_dict_usr[index].type == REAL) {
             flt = (float *)(var_dict_usr[index].addr);
             sprintf(string, "%f", *flt);
          }
      }
      else { 
         index = Parse_Var_Sys(token, temp);
         if (index != -1) { 
            /*
             *   Replace the token with var_dict_sys.
             */
            if (var_dict_sys[index].type == STRING) {
               ptemp = (char *)(var_dict_sys[index].addr);
               strcpy(string, ptemp);
            }
            else if (var_dict_sys[index].type == INTEGER) { 
               ptr = (int *)(var_dict_sys[index].addr);
               sprintf(string, "%d", *ptr);
            }
             else if (var_dict_sys[index].type == REAL) {
                flt = (float *)(var_dict_sys[index].addr);
                sprintf(string, "%f", *flt);
             }
         }
         else {
            /*
             *   Token couldn't be matched to a variable.
             */
            printf("Token does not match to a variable\n");
         }
      }
   }
}






/*(a+out) ------------------------------------------------- */
#ifdef SGI
int  cmd_signal_handler()
#else
void cmd_signal_handler()
#endif
/*(b+out) ------------------------------------------------- 
Procedure Name
cmd_signal_handler - 
Synopsis
Description

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
/*   struct command_list *p;
   char string[CMD_LENGTH];

   p = (struct command_list *) malloc(sizeof(struct command_list));
   if (Read_from_MsgQ(msqid, string) < 0)
      printf("cmd_signal_handler: Something is wrong!!!\n");
   strcpy(p->string, string);
   p->next = NULL;
   if (first_cmd == NULL) {
      first_cmd = last_cmd = p;
   }
   else {
      last_cmd->next = p;
      last_cmd = p;
   }
*/ 
#ifdef SGI
   return TRUE;
#endif
}



#ifdef SGI
int  intr_signal_handler()
#else
void intr_signal_handler()
#endif
{
   if (debug_level >= 20) printf("In intr_signal_handler().........\n");
   Exit();
#ifdef SGI
   return TRUE;
#endif
}


/*(a+out) ------------------------------------------------- */
idle(timeout)
int timeout;
/*(b+out) ------------------------------------------------- 
Procedure Name
idle - 
Synopsis
Description
 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
#ifdef VXWORKS
    wdStart(wid,timeout*sysClkRateGet(),(FUNCPTR)wdtimeout,pid);
#else
    alarm(timeout);
#endif
   is_there_command();
   if(first_cmd == NULL) pause();
}





/*(a+out) ------------------------------------------------- */
#ifdef SGI
int  dummy()
#else
void dummy()
#endif
/*(b+out) ------------------------------------------------- 
Procedure Name
dummy - 
Synopsis
Description

Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
#ifdef SGI
  return TRUE;
#endif
}

#ifdef SGI
int  Busy()
#else
void Busy()
#endif
{
  if(proc_status == ALIVE) proc_status = BUSY;
  else                     proc_status +=1;
  set_value_in_sp("proc_status",&proc_status);
#ifdef SGI
  return TRUE;
#endif
}

#ifdef SGI
int  Set_Interrupt()
#else
void Set_Interrupt()
#endif
{
  interrupt_flag = 1;
#ifdef SGI
  return TRUE;
#endif
}

/*(a+out) ------------------------------------------------- */
GetVariable(arg_string, result_string)
char *arg_string, *result_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
GetVariable - displays the value of a variable  
Synopsis
Description
GetVariable(header, arg_string) displays the value of a variable.   
header: the name of the process which sends the show command,
arg_string: the arguments of the show command.     
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   int  *int_addr, index;
   char *char_addr, rest[CMD_LENGTH];
   char  msg[CMD_LENGTH], temp[CMD_LENGTH];
   float *float_addr;
   Table entry;

   index = Parse_Var_Usr(arg_string, rest);
   if (index != -1) {
      /*
       *   The variable is in var_dict_usr.
       */
      if (var_dict_usr[index].type == INTEGER) {
         int_addr = (int *)var_dict_usr[index].addr; 
         sprintf(result_string, "%s = %d ", arg_string, *int_addr); 
      }
      else if (var_dict_usr[index].type == STRING) {
         char_addr = (char *)var_dict_usr[index].addr; 
         sprintf(result_string, "%s = %s ", arg_string, char_addr); 
      }
      else if (var_dict_usr[index].type == REAL) {
         float_addr = (float *)var_dict_usr[index].addr;
         sprintf(result_string, "%s = %f ", arg_string, *float_addr);
      }
   }
   else {
      index = Parse_Var_Sys(arg_string, rest);
      if (index != -1) {
         /*
          *   The variable is in var_dict_sys.
          */
         if (var_dict_sys[index].type == INTEGER) {
            int_addr = (int *)var_dict_sys[index].addr; 
            sprintf(result_string, "%s = %d ", arg_string, *int_addr); 
         }
         else if (var_dict_sys[index].type == STRING) {
            char_addr = (char *)var_dict_sys[index].addr; 
            sprintf(result_string, "%s = %s ", arg_string, char_addr); 
         }
         else if (var_dict_sys[index].type == REAL) {
            float_addr = (float *)var_dict_sys[index].addr;
            sprintf(result_string, "%s = %f ", arg_string, *float_addr);
         }
      }
      else {
         /*
          *   The variable does not exist.
          */
         sprintf(result_string, "%s = NoSuchVariable ", arg_string);
      }
   }

}



/*(a+out) ------------------------------------------------- */
SetVariable(arg_string, ret_string)
char *arg_string, *ret_string;
/*(b+out) ------------------------------------------------- 
Procedure Name
SetVariable - displays the value of a variable  
Synopsis
Description
SetVariable(arg_string) displays the value of a variable.   
arg_string: the arguments of the show command.  
ret_string: arg_string with the set variable and value stripped out.   
				 
Superconducting Super Collider Laboratory 1.0
(c+out) ------------------------------------------------- */

{
   int  *int_addr, index;
   char *char_addr, new_value[CMD_LENGTH];
   char temp1[CMD_LENGTH], temp2[CMD_LENGTH], eq[3];
   float *float_addr;

   if (debug_level >= 20) printf("Set() = %s\n", arg_string);
   index = Parse_Var_Usr(arg_string, temp1);
   if (index != -1) {
      /*
       *   The variable is in var_dict_usr.
       */
      Parse(temp1, eq, temp2);
      if (strcmp(eq, "=") != 0) 
         printf("Syntax Error: '=' expected.\n");
      Parse(temp2, new_value, temp1);
      if (index >= 0) {
         if (var_dict_usr[index].type == INTEGER) {
            int_addr = (int *)var_dict_usr[index].addr;
            *int_addr = atoi(new_value);
         }
         else if (var_dict_usr[index].type == STRING) {
            char_addr = (char *)var_dict_usr[index].addr; 
            strcpy(char_addr, new_value);
         }
         else if (var_dict_usr[index].type == REAL) {
            float_addr = (float *)var_dict_usr[index].addr;
            *float_addr = stof(new_value);
         }
	 set_value_in_sp(var_dict_usr[index].name,
			 (char *)(var_dict_usr[index].addr));
      }
   }
   else {
      index = Parse_Var_Sys(arg_string, temp1);
      if (index != -1) {
         /*
          *   The variable is in var_dict_sys.
          */
         Parse(temp1, eq, temp2);
         if (strcmp(eq, "=") != 0) 
            printf("Syntax Error: '=' expected.\n");
         Parse(temp2, new_value, temp1);
         if (index >= 0) {
            if (var_dict_sys[index].type == INTEGER) {
               int_addr = (int *)var_dict_sys[index].addr;
               *int_addr = atoi(new_value);
            }
            else if (var_dict_sys[index].type == STRING) {
               char_addr = (char *)var_dict_sys[index].addr; 
               strcpy(char_addr, new_value);
            }
            else if (var_dict_sys[index].type == REAL) {
               float_addr = (float *)var_dict_sys[index].addr;
               *float_addr = (float) stof(new_value);
            }
	    set_value_in_sp(var_dict_sys[index].name,
		      (char *)(var_dict_sys[index].addr));
         }
      }
      else {
         /*
          *   The variable does not exist.
          */
         printf("Error in Set(): No such variable\n");
      }
   }
   strcpy(ret_string, temp1);
}



Wait_for_Reply(process_name, command)
char *process_name, *command;
{
   char string[CMD_LENGTH], pname[CMD_LENGTH];
   char temp1[CMD_LENGTH], temp2[CMD_LENGTH], cmd[CMD_LENGTH];
   char node[100],proc[200],node_t[100],proc_t[200],*tst;
   struct command_list *p, *q;

   Get_Proc_and_Host(process_name,proc,node);
   while (TRUE) {

      /* 
       *   Look at the MsgQ
       */
      if (Read_from_MsgQ(msqid, string) >= 0) { 
         p = (struct command_list *) malloc(sizeof(struct command_list));
         strcpy(p->string, string);
         p->next = NULL;
         if (first_cmd == NULL) {
            first_cmd = last_cmd = p;
         }
         else {
            last_cmd->next = p;
            last_cmd = p;
         }
      }
    
#ifdef VXWORKS
      wdStart(wid,100,(FUNCPTR)wdtimeout,pid);
#else
      alarm(1);
#endif
      q = NULL;
      p = first_cmd;
      while (p != NULL) {
         strcpy(string, p->string);
         Parse(string, pname, temp1);
         Get_Proc_and_Host(pname,proc_t,node_t);
         if (strcmp(proc_t, proc) == 0) {
	    if((node[0] == '\0') || (node_t[0] == '\0') ||
	       (strncmp(node,node_t,strlen(node)) == 0) ||
	       (strncmp(node,node_t,strlen(node_t)) == 0)) {
              Parse(temp1, cmd, temp2);
              if (strcasecmp(cmd, command) == 0) {
               /*
                * Remove the command from the command list.
                */
               if (q == NULL) {
                  first_cmd = p->next;
                  free((char *)p);
               }
               else {
                  q->next = p->next;
                  free((char *)p);
               }
               Process_Command(string, NULL);
               return;
              }
	    }
         }
         q = p;
         p = p->next; 
      }
      idle(1);
   }
   
}

float stof(str)
char *str;
{
   float result, frac;
   int   dot, len1, i;

   i = 0;
   while ((i<strlen(str)) & (str[i] != '.')) i++;
   len1 = i;

   result = 0.0;
   for (i=0; i<len1; i++) result = result*10 + str[i]-'0';
   frac = 1;
   for (i=len1+1; i<strlen(str); i++) {
      frac /= 10.0;
      result += frac * (str[i]-'0');
   }
   return (result);
}

#ifdef VXWORKS

void wdtimeout(param)
int param;
{
  kill(param,SIGALRM); 
}
#endif
