/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      June 20, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <vxWorks.h>
#include <msgQLib.h>
#define MAXMSG 50
#define MAXMSGSZ 200
#else
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>
#include <stdio.h>
#define MSG_Q_ID int
#endif

#ifndef VXWORKS
struct msgbuf *rcvbuf, *sndbuf;
int rcvbuf_define=1;
int sndbuf_define=1;
#endif

#include "template.h"

/*(a+out) ------------------------------------------------- */
int Create_MsgQ(msqid)
MSG_Q_ID *msqid;
/*(b+out) -------------------------------------------------
Procedure Name
Create_MsgQ - creates a message queue 
Synopsis
Description
Create_MsgQ(msqid) creates a message queue.   
msqid  : message queue id of the new message queue.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  *msqid = msgQCreate(MAXMSG,MAXMSGSZ,MSG_Q_FIFO);
  if(*msqid == NULL) return ERROR;
#else
   key_t key;
   int opperm, opperm_flags; 

   key = 0;
   opperm = 0666;
   opperm_flags = (opperm | IPC_CREAT);
   *msqid = msgget(key, opperm_flags);
   if (rcvbuf_define) {
     rcvbuf = (struct msgbuf *) malloc(sizeof(long) + MAX_MSG_SIZE + 1);
     rcvbuf_define=0;
   }
   if (sndbuf_define) {
     sndbuf = (struct msgbuf *) malloc(sizeof(long) + MAX_MSG_SIZE + 1);
     sndbuf_define=0;
   }
   if (*msqid == -1) return ERROR; 
#endif
   return SUCCESS;
}




/*(a+out) ------------------------------------------------- */
int Delete_MsgQ(msqid)
MSG_Q_ID msqid;
/*(b+out) -------------------------------------------------
Procedure Name
Delete_MsgQ - deletes a message queue
Synopsis
Description 
Delete_MsgQ(msgqid) deletes a message queue. 
msqid: message queue id of the MsgQ to be detached. 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  if (msgQDelete(msqid) == ERROR) return ERROR;
#else
  if (msgctl(msqid, IPC_RMID, NULL) == -1) return ERROR; 
#endif
  return SUCCESS; 
}




/*(a+out) ------------------------------------------------- */
int Write_to_MsgQ(msqid, message)
MSG_Q_ID msqid;
char *message;
/*(b+out) -------------------------------------------------
Procedure Name
Write_to_MsgQ - adds a string to a MsgQ 
Synopsis
Description
Write_to_MsgQ(msqid, message) adds a string to a MsgQ. 
msqid  : message queue id of the MsgQ,   
message: the string that will be added to the MsgQ.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  STATUS rtrn;

  rtrn = msgQSend(msqid, message,strlen(message),10,MSG_PRI_NORMAL);
  if(rtrn == ERROR) return ERROR;
  return strlen(message);
#else
   long msgtyp;
   int msgsz, msgflg, rtrn;

   if (sndbuf_define) {
     sndbuf = (struct msgbuf *) malloc(sizeof(long) + MAX_MSG_SIZE + 1);
     sndbuf_define=0;
   }
   msgsz = strlen(message);
   strcpy(sndbuf->mtext, message);
   msgflg = IPC_NOWAIT;
   sndbuf->mtype = MSG_TYPE;

   rtrn = msgsnd(msqid, sndbuf, msgsz, msgflg);  
   if (rtrn == -1) return ERROR;
   else return rtrn; 
#endif
}




/*(a+out) ------------------------------------------------- */
int Read_from_MsgQ(msqid, message)
MSG_Q_ID msqid;
char *message;
/*(b+out) -------------------------------------------------
Procedure Name
Read_from_MsgQ - reads a string from a MsgQ
Synopsis
Description
Read_from_MsgQ(msqid, message) reads a string from a MsgQ.
msqid  : message queue id of the MsgQ, 
message: the string to where the message read will be placed. 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  int rtrn;

  rtrn = msgQReceive(msqid,message,MAXMSGSZ,NO_WAIT);
  if(rtrn == -1) return ERROR;
  message[rtrn] = '\0';
  return rtrn;
#else
   long msgtyp;
   int msgsz, msgflg, rtrn, i;

   if (rcvbuf_define) {
     rcvbuf = (struct msgbuf *) malloc(sizeof(long) + MAX_MSG_SIZE + 1);
     rcvbuf_define=0;
   }
   msgsz = MAX_MSG_SIZE;
   msgflg = IPC_NOWAIT;
   msgtyp = MSG_TYPE;
   rtrn = msgrcv(msqid, rcvbuf, msgsz, msgtyp, msgflg);
   if (rtrn == -1) return ERROR;
   else {
      for (i = 0; i < rtrn; i++) message[i] = rcvbuf->mtext[i];
      message[i] = '\0';
      return rtrn; 
   }
#endif
}

