/*---------------------------------------------------------------*/
/*                                                               */
/*      Cengiz ERBAS                                             */
/*      Superconducting Super Collider Laboratory                */
/*      August 7, 1992                                           */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <vxWorks.h>
#include <semLib.h>
#else
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#define SEM_ID int
#endif
#include "template.h" 

#if defined(HP) || defined(DEC) || defined(HPRT)
   union semun {
      int val;
      struct semid_ds *buf;
      ushort *array;
   };
#endif

#if !defined(VXWORKS)
static struct semid_ds semid_ds;
#endif

/*(a+out) ------------------------------------------------- */
int Create_Sem(semid)
SEM_ID *semid;
/*(b+out) -------------------------------------------------
Procedure Name
Create_Sem - creates a semaphore
Synopsis
Description
Create_Sem(semid) creates a semaphore.
semid  : semaphore id of the new semaphore.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  if((*semid = semBCreate(SEM_Q_FIFO, SEM_FULL)) == NULL) return ERROR;
#else
   key_t key;
   int nsems, semflg;

   key = IPC_PRIVATE;
   nsems = 1;
   semflg = 0666;
   if ((*semid = semget(key, nsems, semflg)) == -1) return ERROR;
#endif
   return SUCCESS; 
}




/*(a+out) ------------------------------------------------- */
int Update_Sem_Wait(semid, value)
SEM_ID semid;
int  value;
/*(b+out) -------------------------------------------------
Procedure Name
Update_Sem_Wait - increments the semaphore by value
Synopsis
Description
Update_Sem_Wait(semid, value) increments the semaphore by 
value. If the operation is not successful , it waits. 
semid  : semaphore id of the semaphore to be updated. 
value  : the integer that will be added to the semaphore.  

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  if(value == -1) semTake(semid, WAIT_FOREVER);
  else if (value == 1) semGive(semid);
#else
   struct sembuf *sops;
   int retval, nsops = 1;

   sops = (struct sembuf *) malloc (sizeof(struct sembuf));
   sops[0].sem_num = 0;
   sops[0].sem_op = value;
   sops[0].sem_flg = 0;
   if (semop(semid, sops, nsops) == -1) {
	perror("semop failed");
      free(sops);

      return ERROR;
   }
   free(sops);
#endif
   return SUCCESS;
}




/*(a+out) ------------------------------------------------- */
int Update_Sem_NoWait(semid, value)
SEM_ID semid;
int  value;
/*(b+out) -------------------------------------------------
Procedure Name
Update_Sem_NoWait - increments the semaphore by value
 
Synopsis
Description
Update_Sem_NoWait(semid, value) increments the semaphore by
value. If the operation is not successful, it returns -1.
semid  : semaphore id of the semaphore to be updated. 
value  : the integer that will be added to the semaphore.
 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  if(value == -1) semTake(semid, NO_WAIT);
  else if (value == 1) semGive(semid);
  return SUCCESS;
#else
   struct sembuf *sops;
   int retval, nsops = 1;

   sops = (struct sembuf *) malloc (sizeof(struct sembuf));
   sops[0].sem_num = 0;
   sops[0].sem_op = value;
   sops[0].sem_flg = IPC_NOWAIT;
   retval = semop(semid, sops, nsops);
   free(sops);
   return(retval);
#endif
}





/*(a+out) ------------------------------------------------- */
void Display_Sem(semid)
SEM_ID semid;
/*(b+out) -------------------------------------------------
Procedure Name
Display_Sem - displays the value of a semaphore 
Synopsis
Description
Display_Sem(semid) displays the value of a semaphore.
semid  : semaphore id of the semaphore to be displayed. 
         
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  int idList[10],num;

  num = semInfo(semid,idList,10);
  printf("semaphore value %d\n",num);
#else
   union semun arg;

   arg.buf = &semid_ds;
   if (semctl(semid, 0, IPC_STAT, arg) == -1) {
      perror("semctl failed: ");
      exit(1);
   }
   if ((arg.array = (u_short *) malloc(sizeof(ushort))) == NULL) {
      printf("Can't allocate memory\n");
      exit(1);
   }
   if (semctl(semid, 0, GETALL, arg) == -1) {
      perror("semctl failed: ");
      exit(1);
   }
   printf("semaphore value = %d\n", arg.array[0]);
#endif
}
/*(a+out) ------------------------------------------------- */
int Getval_Sem(semid)
SEM_ID semid;
/*(b+out) -------------------------------------------------
Procedure Name
Getval_Sem - returns the value of a semaphore 
Synopsis
Description
Getval_Sem(semid) gets the value of a semaphore.
semid  : semaphore id of the semaphore to be displayed. 
         
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  int idList[10],num;

  num = semInfo(semid,idList,10);
  return(num);
#else
   union semun arg;

   arg.buf = &semid_ds;
   if (semctl(semid, 0, IPC_STAT, arg) == -1) {
      perror("semctl failed: ");
      exit(1);
   }
   if ((arg.array = (u_short *) malloc(sizeof(ushort))) == NULL) {
      printf("Can't allocate memory\n");
      exit(1);
   }
   if (semctl(semid, 0, GETALL, arg) == -1) {
      perror("Getval_Sem: semctl failed: ");
      exit(1);
   }
   return (arg.array[0]);
#endif
}
/*(a+out) ------------------------------------------------- */
int Setval_Sem(semid,value)
SEM_ID semid;
int value;
/*(b+out) -------------------------------------------------
Procedure Name
Setval_Sem - sets the value of a semaphore 
Synopsis
Description
Setval_Sem(semid) sets the value of a semaphore.
semid  : semaphore id of the semaphore to be displayed. 
         
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
/*** This has no meaning in VxWorks ***/
#else
   union semun arg;

   arg.buf = &semid_ds;
   if (semctl(semid, 0, IPC_STAT, arg) == -1) {
      perror("semctl failed: ");
      exit(1);
   }
   if ((arg.array = (u_short *) malloc(sizeof(ushort))) == NULL) {
      printf("Can't allocate memory\n");
      exit(1);
   }
   arg.val=value;
   if (semctl(semid, 0, SETVAL, arg) == -1) {
      perror("Setval_Sem: semctl failed: ");
      exit(1);
   }
#endif
   return (SUCCESS);
}





/*(a+out) ------------------------------------------------- */
int Delete_Sem(semid)
SEM_ID semid;
/*(b+out) -------------------------------------------------
Procedure Name
Delete_Sem - detaches a semaphore 
 
Synopsis
Description
Delete_Sem(semid) detaches a semaphore.  
semid  : semaphore id of the semaphore to be removed.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
#ifdef VXWORKS
  if(semDelete(semid) == ERROR) return ERROR;
#else
   union semun arg;
   arg.buf = &semid_ds;
    arg.val = 0;

   if (semctl(semid, 0, IPC_RMID, arg) == -1) {
	perror("Delete_Sem: semctl error ");
	return ERROR;
   }
#endif
   return SUCCESS; 
}
