/* /////////////////////////////////////////////////////////////
//
// file:    libutil.C
// author:  C. Timmermans
// version: 1.0
// date:    9/24/92
/////////////////////////////////////////////////////////// */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#ifdef VXWORKS
struct envvar {
char *name;
char *value;
};
extern int n_vx_env_var;
extern struct envvar vx_env_var[];
char *getenv();
#else
#include <sys/wait.h>
#include<string.h>
#include<sys/time.h>
#include<stdio.h>
#include<stdlib.h>

#ifdef SGI 
#define _BSD_SIGNALS 
#endif 
#include<signal.h>
#endif

#include "template.h"


int wait_for_child(st)
int *st;
{
#ifdef HP
	return(wait(st));
#endif
#ifdef SUN
	union wait status;
	int pid;
	
	pid = wait(&status);
	*st = status.w_status;
	return(pid);
#endif
#ifdef DEC
	union wait status;
	int pid;
	
	pid = wait(&status);
	*st = status.w_status;
	return(pid);
#endif
}


int decode_filename(partial,full)
char *partial;
char *full;
/*///////////////////////////////////////////////////////////
// checks if partial starts with an environment variable,
// if so it tries to decode it.
// The full filename is set in full. If the decoding fails
// full will be the null-string.
// Return codes:
//        -1 The structure of partial is NOT ok
//        -2 Partial does NOT start with a '$' or a '/'
//        -3 The environment variable cannot be decoded
//         0 all is ok
////////////////////////////////////////////////////////////*/
{
	int i_min,i_max;
	char env_name[80],*env_val;

	full[0] = '\0';
	if(partial[0] == '/') {
	  strcpy(full,partial);
	  return(0);
	}
	if(partial[0] == '$') {
	  if(partial[1] == '(') {
/*//////////////////////////////////////////////////////
// The environment variable is surrounded by brackets
//////////////////////////////////////////////////////*/
	    i_min = 2;
	    i_max = i_min;
	    while(partial[i_max] != ')') {
	      if (partial[i_max] == '\0') return(-1);
	      i_max++;
	    }
	    i_max--;
	  }
	  else {
/*////////////////////////////////////////////////////////
// The environment variable is NOT surrounded by brackets
////////////////////////////////////////////////////////*/
	    i_min = 1;
	    i_max = i_min;
	    while(partial[i_max] != '/') {
	      if (partial[i_max] == '\0') return(-1);
	      i_max++;
	    }
	    i_max--;
	  }
/*///////////////////////////////////////////////////////////////
// Fill and try to decode the environment variable
//////////////////////////////////////////////////////////////*/
	  strncpy(env_name,&(partial[i_min]),i_max-i_min+1);
	  env_name[i_max-i_min+1] = '\0';
	  if((env_val = getenv(env_name)) == NULL) return(-3);
/*/////////////////////////////////////////////////////////
// fill the full filename
/////////////////////////////////////////////////////////*/
	  while(partial[i_max] != '/') {
	    if (partial[i_max] == '\0') return(-1);
	    i_max++;
	  }
	  strcpy(full,env_val);
	  strcat(full,&(partial[i_max]));
	  return(0);
	}
	return(-2);
}

void Modified_Storage(par,stor)
char *par,*stor;
/*////////////////////////////////////////////////////////////////
// Storage parameters are to be modified by several processes. If the
// appropriate message is sent in the standard way this routine
// handles that.
//////////////////////////////////////////////////////////////*/
{
	char wrd[100];

#ifdef _DEBUG
	  printf("Modify storage: %s\n",par);
#endif
	if(sscanf(par,"%s",wrd) > 0)
	  strcpy(stor,wrd);
	else
	  stor[0] = '\0';
}

#ifdef VXWORKS
char *getenv(name)
char *name;
{
  int i;
  
  for(i=0;i<n_vx_env_var;i++) {
    if(strcmp(name,vx_env_var[i].name) == 0) return(vx_env_var[i].value);
  }
  return(NULL);
}

#ifdef NEVER
int atoi(string)
char *string;
{
   int value, i;

   value = 0;
   for (i=0; i<strlen(string); i++) {
      value = 10*value + (string[i] - '0');
   } 
   return(value);
}
#endif

int strcasecmp(string1, string2)
char *string1, *string2;
{
   char temp1[255], temp2[255];

   touppercase(string1, temp1); 
   touppercase(string2, temp2); 
   return(strcmp(temp1, temp2));
}


int strncasecmp(string1, string2, n)
char *string1, *string2;
int n;
{
   char temp1[255], temp2[255];

   touppercase(string1, temp1); 
   touppercase(string2, temp2); 
   return(strncmp(temp1, temp2, n));
}


touppercase(source, target)
char *source, *target;
{
   int i;

   for (i=0; i<strlen(source)+1; i++)
      if ((source[i] >= 'a') && (source[i] <= 'z'))
         target[i] = source[i] + 'A' - 'a';
      else
         target[i] = source[i];   
}
#endif
