/* ////////////////////////////////////////////////////////
//
// file:    runpar_win.C
// author:  C. Timmermans
// version: 2.0
// date:    7/29/93
/////////////////////////////////////////////////////// */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include"operator.h"
#include"runpar.h"

#include <X11/Xlib.h>
#include<sys/types.h>
#include<time.h>
#include<sys/time.h>
#include<string.h>
#include<strings.h>

extern int i_message;
extern char runcontrol[];
char collector[150];
extern char recorder[];

struct timeval t_getdata;


Window run_par;
GC rp_GC;

Window rp_bt[MAX_N_RP_BUT];
GC     rp_btGC[MAX_N_RP_BUT];

char value_bt[MAX_N_RP_BUT][100],text_bt[MAX_N_RP_BUT][100];
int rp_ysiz;

extern XEvent report;

/* //////////////////////////////////////////////////////////////////
// Init_win_runpar: initializes the runpar window, together
// with the variables needed for this window
///////////////////////////////////////////////////////////////// */
Init_win_runpar()
{
  int i,but_x,but_y,opt;

#ifdef _DEBUG
  printf("Starting to initialize the Runpar window\n");
#endif
  if(n_rp_but > MAX_N_RP_BUT) n_rp_but = MAX_N_RP_BUT;

  rp_ysiz = 30+40*((n_rp_but + 1)/2);
  gui_window(&run_par,&rp_GC,0,600,rp_ysiz,15,225,4,0,"Run Parameters");
  for(i = 0; i<n_rp_but;i++){
    but_x = 110+310*(i%2);
    but_y = 25 + 40*(i/2);
    if(rp_but[i].rc_mod == YES) opt = 1;
    else                        opt = 0;
    gui_window(&(rp_bt[i]),&(rp_btGC[i]),&run_par,160,30,but_x,but_y,1,opt,"");
  }
  gui_display(&run_par);
  for(i = 0; i<n_rp_but;i++) gui_display(&(rp_bt[i]));
  t_getdata.tv_sec = 0;
}

Update_win_runpar()
/* ////////////////////////////////////////////////
// checks if the screen is up to date with the runpar
// and storage parameters. In this way the window-manager
// needs only to be 'signalled' to make an update; otherwise
// elaborate messages need to be sended
///////////////////////////////////////////////////// */
{
  struct timeval tv;
  struct timezone tz;
  
  gettimeofday(&tv,&tz);
  if((tv.tv_sec - t_getdata.tv_sec) > DATATIME) {
    get_run_parameters();
    gettimeofday(&tv,&tz);
    t_getdata.tv_sec = tv.tv_sec;
  }
}

Execute_win_runpar()
/* ////////////////////////////////////////////////////////
// After receiving a message execute screen is called
/////////////////////////////////////////////////////// */
{
  int i;

/* /////////////////////////////////////////////////////////
// Update the window that has been changed through the
// screen-actions of the operator.
///////////////////////////////////////////////////////// */
  if(report.xkey.window == run_par)          update_run_par();
  else {
    for(i = 0;i<n_rp_but;i++){
      if(report.xkey.window == rp_bt[i]){
	update_rp_but(i);
	return;
      }
    }
  }
}

update_run_par()
/* ///////////////////////////////////////////////////
// The run_par window reacts to Expose-events only
///////////////////////////////////////////////// */
{
  int i,tx_x,tx_y;

  switch(report.type) {
  case Expose:
    gui_text(&run_par,&rp_GC,110,15,"Collector:");
    gui_text(&run_par,&rp_GC,180,15,collector);
    for(i = 0; i<n_rp_but;i++){
      tx_x = 10+310*(i%2);
      tx_y = 45 + 40*(i/2);
      gui_text(&run_par,&rp_GC,tx_x,tx_y,rp_but[i].txt);
    }
    break;
  }
}

update_rp_but(but)
int but;
{
  int i;
  float f;
  
  switch(report.type) {
  case Expose:
    if(rp_but[but].type == STRING) strcpy(text_bt[but],value_bt[but]);
    else if(rp_but[but].type == REAL) {
      memcpy(&f,value_bt[but],sizeof(float));
      sprintf(text_bt[but],"%g",f);
    }
    else {
      memcpy(&i,value_bt[but],sizeof(int));
      if(rp_but[but].type == INT_DEC) sprintf(text_bt[but],"%d",i);
      else if(rp_but[but].type == INT_HEX) sprintf(text_bt[but],"0x%x",i);
      else if(rp_but[but].type == INT_CONV) { 
	if (rp_but[but].conversion(&i,text_bt[but],0) != 0) 
	  strcpy(text_bt[but],"!!ERROR!!");
      }
      else break;
    }
    gui_text(&(rp_bt[but]),&(rp_btGC[but]),5,20,text_bt[but]);
    break;
  case KeyPress:
    if(rp_but[but].rc_mod == NO) break;
    if(i_message == MSG_RUN) {
      if( gui_string(report,&(rp_bt[but]),&(rp_btGC[but]),text_bt[but],5,20,
		     0, 10) == 1) {
	if(rp_but[but].type == INT_DEC) {
	  if(sscanf(text_bt[but],"%d",&i) > 0){
	    memcpy(value_bt[but],&i,sizeof(int));
	  }
	}
	else if(rp_but[but].type == INT_HEX) { 
	  if(sscanf(text_bt[but],"0x%x",&i) > 0) 
	    memcpy(value_bt[but],&i,sizeof(int));
	}
	else if(rp_but[but].type == REAL) { 
	  if(sscanf(text_bt[but],"%g",&f) > 0) 
	    memcpy(value_bt[but],&f,sizeof(float));
	}
	else if(rp_but[but].type == STRING) strcpy(value_bt[but],text_bt[but]);
	gui_clear(&(rp_bt[but]));
	report.type = Expose;
	update_rp_but(but);
      }
    } 
  }
}

get_run_parameters()
{
  int i,i_change,retval;
  short s;
  char tmp[100];
  XEvent r_temp;

  memcpy(&r_temp,&report,sizeof(XEvent));
  report.type = Expose;
  if(read_value_from_sp(runcontrol,"collector",&s,collector) != 0) 
    strcpy(collector,"collector");
  if(read_value_from_sp(runcontrol,"recorder",&s,recorder) != 0) 
    strcpy(recorder,"recorder");
  if((i_message != MSG_RUN)) {
    for(i = 0;i<n_rp_but;i++){
      if(strcmp(rp_but[i].proc,"runcontrol") == 0)
	retval = read_value_from_sp(runcontrol,rp_but[i].variable,&s,tmp);
      else if(strcmp(rp_but[i].proc,"collector") == 0)
	retval = read_value_from_sp(collector,rp_but[i].variable,&s,tmp);
      else if(strcmp(rp_but[i].proc,"recorder") == 0)
	retval = read_value_from_sp(recorder,rp_but[i].variable,&s,tmp);
      else
	retval = read_value_from_sp(rp_but[i].proc,rp_but[i].variable,&s,tmp);
      i_change = 0;
      if(retval == 0){
	if(rp_but[i].type == STRING) {
	  if(strcmp(tmp,value_bt[i]) != 0) {
	    strcpy(value_bt[i],tmp);
	    i_change = 1;
	  } 
	}
	else if(rp_but[i].type == REAL) {
	  if(memcmp(tmp,value_bt[i],sizeof(float)) != 0) {
	    memcpy(value_bt[i],tmp,sizeof(float));
	    i_change = 1;
	  } 
	}
	else {
	  if(memcmp(tmp,value_bt[i],sizeof(int)) != 0) {
	    memcpy(value_bt[i],tmp,sizeof(int));
	    i_change = 1;
	  }
	}
      }
      if(i_change == 1) {
	gui_clear(&(rp_bt[i]));
	update_rp_but(i);
      }
    }
  }
  memcpy(&report,&r_temp,sizeof(XEvent));
}

