/* ////////////////////////////////////////////////////////
//
// file:    storage_win.C
// author:  C. Timmermans
// version: 2.0
// date:    3/10/93
/////////////////////////////////////////////////////// */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include"operator.h"

#include <X11/Xlib.h>
#include<sys/types.h>
#include<string.h>
#include<strings.h>
#include<time.h>
#include<sys/time.h>

char sinkdir[150],sinkdev[100],sinklab[100];
char new_sinkdir[150],new_sinkdev[100];
char recorder[150];
char old_recorder[150];

char str_fname[80], str_device[100];

struct timeval t_getstor;

extern int i_message;
extern char runcontrol[];

Window storage, st_Fname, st_Device;
GC st_GC, st_FnameGC, st_DeviceGC;

extern XEvent report;

/* /////////////////////////////////////////////////////////////////////
// Initializes the storage-window with appropriate variables
//////////////////////////////////////////////////////////////////// */
Init_win_storage()
{
#ifdef _DEBUG
	printf("Starting to initialize the storage window\n");
#endif
	gui_window(&storage,&st_GC,0,600,100,15,80,4,0,"Storage");
	gui_window(&st_Fname,&st_FnameGC,&storage,400,20,130,40,1,1,"");
	gui_window(&st_Device,&st_DeviceGC,&storage,400,20,130,70,1,1,"");
	gui_display(&storage);
	gui_display(&st_Fname);
	gui_display(&st_Device);
	str_fname[0] = '\0';
	sinkdir[0] = '\0';
	t_getstor.tv_sec = 0;
	strcpy(recorder,"recorder");
	strcpy(old_recorder,recorder);
}
Update_win_storage()
/* ////////////////////////////////////////////////
// checks if the screen is up to date with the runpar
// and storage parameters. In this way the window-manager
// needs only to be 'signalled' to make an update; otherwise
// elaborate messages need to be sended
///////////////////////////////////////////////////// */
{
	XEvent r_temp;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv,&tz);
	if((tv.tv_sec - t_getstor.tv_sec) > STORTIME) {
	  get_storage_parameters();
	  gettimeofday(&tv,&tz);
	  t_getstor.tv_sec = tv.tv_sec;
	}
	memcpy(&r_temp,&report,sizeof(XEvent));
	report.type = Expose;
	if(strcmp(recorder,old_recorder) != 0) {
	  strcpy(old_recorder,recorder);
	  gui_clear(&storage);
	  update_storage();
	}
	if(strcmp(new_sinkdir,sinkdir) != 0) {
	  strcpy(sinkdir,new_sinkdir);
	  strcpy(str_fname,sinkdir);
	  gui_clear(&st_Fname);
	  update_st_Fname();
	}
	if(strcasecmp(sinkdir,"tape") == 0) {
	  if(strcmp(sinklab,"device") != 0){
	    strcpy(sinklab,"device");
	    gui_clear(&storage);
	    update_storage();
	  }
	}
	else if(strcasecmp(sinkdir,"dummy") == 0) {
	  if(strcmp(sinklab,"device/directory") != 0){
	    strcpy(sinklab,"device/directory");
	    gui_clear(&storage);
	    update_storage();
	  }
	}
	else{
	  if(strcmp(sinklab,"directory") != 0){
	    strcpy(sinklab,"directory");
	    gui_clear(&storage);
	    update_storage();
	  }
	}
	if(strcmp(new_sinkdev,sinkdev) != 0) {
	  strcpy(sinkdev,new_sinkdev);
	  strcpy(str_device,sinkdev);
	  gui_clear(&st_Device);
	  update_st_Device();
	}
	memcpy(&report,&r_temp,sizeof(XEvent));
}
Execute_win_storage()
{
  if(report.xkey.window == storage)          update_storage();
  else if(report.xkey.window == st_Fname)    update_st_Fname(); 
  else if(report.xkey.window == st_Device)    update_st_Device(); 
}

update_storage()
/* /////////////////////////////////////////////////
// The storage-window only reacts to Expose events
///////////////////////////////////////////////// */
{
        if(i_message == MSG_STOR){
	  strcpy(sinklab,"device/directory");
	}
	switch(report.type) {
	case Expose:
	  gui_text(&storage,&st_GC,10,55,"Event sink"); 
	  gui_text(&storage,&st_GC,10,85,sinklab); 
	  gui_text(&storage,&st_GC,100,20,"Recorder:"); 
	  gui_text(&storage,&st_GC,165,20,recorder); 
	}
}
update_st_Fname()
/* ////////////////////////////////////////////////////////
// The file-name of the storage-window can be modified
// during the initialization-phase of a run; therefore
// this window reacts to a keypress.
///////////////////////////////////////////////////////// */
{
	switch(report.type) {
	case Expose:
	  gui_text(&st_Fname,&st_FnameGC,5,15,str_fname); 
	  break;
	case KeyPress:
	  if(i_message == MSG_STOR) {
	    if( gui_string(report,&st_Fname,&st_FnameGC,str_fname,5,15,
                    0, 79) == 1) {
	      gui_text(&st_Fname,&st_FnameGC,5,15,str_fname);
	    }
	  }
	} 
}

update_st_Device()
/* ////////////////////////////////////////////////////////
// The file-name of the storage-window can be modified
// during the initialization-phase of a run; therefore
// this window reacts to a keypress.
///////////////////////////////////////////////////////// */
{
	switch(report.type) {
	case Expose:
	  gui_text(&st_Device,&st_DeviceGC,5,15,str_device); 
	  break;
	case KeyPress:
	  if(i_message == MSG_STOR) {
	    if( gui_string(report,&st_Device,&st_DeviceGC,str_device,5,15,
                    0, 79) == 1) {
	      gui_text(&st_Device,&st_DeviceGC,5,15,str_device);
	    }
	  }
	} 
}

get_storage_parameters()
{
  short s;
  
  if(i_message != MSG_STOR) {
    if (read_value_from_sp(runcontrol,"recorder",&s,recorder) != 0)
      strcpy(recorder,"recorder");
    if(read_value_from_sp(recorder,"eventsink",&s,new_sinkdir) != 0){
      strcpy(new_sinkdir,sinkdir);
      return;
    }
    if(strcasecmp(new_sinkdir,"dummy") == 0) {
      strcpy(new_sinkdev,"none");
      return;
    }
    if(strcasecmp(new_sinkdir,"tape") == 0) {
      read_value_from_sp(recorder,"device",&s,new_sinkdev);
      return;
    }
    if(read_value_from_sp(recorder,"dest_dir",&s,new_sinkdev) != 0)
      strcpy(new_sinkdev,sinkdev);
  }
  else
    strcpy(new_sinkdir,sinkdir);
}
