/* //////////////////////////////////////////////////////////////////////////
//
// file:    xpc.C
// author:  C. Timmermans
// version: 1.0
// date:    10/16/92
///////////////////////////////////////////////////////////////////////// */

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include "vxWorks.h"
#include "vxipc.h"
#endif

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <signal.h>
#include <stdlib.h>
#include "template.h"
#include "xpc.h"

#ifndef VXWORKS
#include <sys/time.h>
#include <sys/ipc.h>
#include <strings.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include "sdc_pack_msg_c.h"
#endif

void Init_Process();
void Dead_Process();
void check_all_queues();
void send_other_xpc();
void Alive();
void Check_Process();
void No_Event();
extern char *get_name_from_args();

#define YES 1
#define NO  0
#define MSGTIME 60
#define RESPONSE 2

#define MEM_KEY 49456
/*************************************************
 The numbers below must be less than the system limits !!
**************************************************/
#define MAX_MESSAGES 20
#define MAX_MSQBYTES 8000
/*****************************************************/
/* some variables for general use                    */
/*****************************************************/

struct progcheck{
#ifdef VXWORKS
        time_t t_last;
#else
	struct timeval	t_last;
#endif
	int t_msg;
	int state;
	int restart;
	int no_check;
};
struct progcheck *checkprg;
int *n_checkprg;
int mem_id;

extern char this_node[];
extern char ownpname[];
Table Common_Table[TABLE_SIZE];

/**********************************************************/
#ifdef VXWORKS
xpc(argv1,argv2,argv3,argv4,argv5,argv6,argv7,
          argv8,argv9,argv10) 
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,*argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
		    argv9,argv10}; 
  int argc = 11;
#else
main(argc, argv)
int argc;
char **argv;
{
#endif
  char string[500];
  Table entry;
  int index,result;

#ifdef VXWORKS
  argv[0] = "xpc";
#endif

  if(strcmp(get_name_from_args(argc,argv),"xpc") != 0) exit(1);
  if( (result = Template_Init(get_name_from_args(argc,argv))) == -1) {
/*** The stuff is there and ALIVE on this node (according to table); kill it 
     and remove it's queue's (might or might not be the decent thing to do) **/
    index = Search_Table(ownpname);
    if(index < 0) exit(0);
    Read_from_Table(&entry,index);
    if(entry.status != DEAD){
#ifdef _DEBUG
      printf("Killing %s before restarting completely\n",ownpname);
#endif
      kill(entry.process_id,9);
      Delete_MsgQ(entry.msgq_id);
      entry.status = DEAD;
      Write_to_Table(&entry,index);
      Add_Entry_to_Table();
      Init_Status_Path();
    }
    else exit(0);
  }
  else if (result < 0) {
/*** The stuff is there and ALIVE on some other node (according to table); 
     Note it being dead so that this process can be listed in the table. ***/
    index = Search_Table(ownpname);
    if(index >= 0){
      Read_from_Table(&entry,index);
      entry.status = DEAD;
      entry.node[0] = '\0';
      Write_to_Table(&entry,index);
    }
    Add_Entry_to_Table();
    Init_Status_Path();
  }
  Init_Process();
  Template_Main(NULL,NULL,No_Event);
}

/***********************************************************/
void Init_Process() /*perform startup operations*/
{
  int i;
  char tmp[100];
/* /////////////////////////////////////////////////////////////
// The important parameters are in shared memory in case 
// the XPC dies. The new XPC takes over these parameters
//////////////////////////////////////////////////////////// */
  get_xpc_shm();
/* //////////////////////////////////////////////////////////
// If there are processes restarted before the XPC
////////////////////////////////////////////////////////// */
  update_xpc_shm();
/*** Set the correct PROC_STATUS for all processes on this node ***/
  sprintf(tmp,"Set proc_status = %d",ALIVE);
  for(i=0;i<*n_checkprg;i++){
    if((Common_Table[i].node[0] == '\0') ||
       (strncmp(Common_Table[i].node,this_node,
		strlen(Common_Table[i].node)) == 0) ||
       (strncmp(Common_Table[i].node,this_node,strlen(this_node)) == 0) ){
      if(strcmp(Common_Table[i].process_name,ownpname) != 0)
	Send_Message(Common_Table[i].process_name,tmp);
    }
  }
}
/***********************************************************/
get_xpc_shm()
{
#ifdef _DEBUG
  printf("Opening the shared memory\n");
#endif
  mem_id = shmget(MEM_KEY,(50*sizeof(struct progcheck)+sizeof(int)),
		  IPC_CREAT|IPC_EXCL|00666);
  if(mem_id == -1){
    mem_id = shmget(MEM_KEY,(50*sizeof(struct progcheck)+sizeof(int)),
		    00666);
    if(mem_id == -1) {
      printf("Cannot create the XPC shared memory\n");
      exit(1);
    }
    checkprg = (struct progcheck *)shmat(mem_id,0,SHM_RND);
    n_checkprg = (int *)((char *)checkprg + 50*sizeof(struct progcheck));
  }
  else {
    checkprg = (struct progcheck *)shmat(mem_id,0,SHM_RND);
    n_checkprg = (int *)((char *)checkprg + 50*sizeof(struct progcheck));
    *n_checkprg = 0;
  }
}
/*********************************************************************/
update_xpc_shm()
{
  int i;
#ifdef VXWORKS
  time_t tv;
#else
  struct timeval tv;
  struct timezone tz;
#endif
  Table *entry;

  i = Get_Table(Common_Table);
#ifdef VXWORKS
  time(&tv);
#else
  gettimeofday(&tv,&tz);
#endif
  while(*n_checkprg < i) {
    entry = &(Common_Table[*n_checkprg]);
#ifdef VXWORKS
    checkprg[*n_checkprg].t_last = tv;
#else
    memcpy(&(checkprg[*n_checkprg].t_last),&tv,sizeof(struct timeval));
#endif
    checkprg[*n_checkprg].restart = NO;
    checkprg[*n_checkprg].t_msg = MSGTIME;
    checkprg[*n_checkprg].state = entry->status;
    checkprg[*n_checkprg].no_check = NO;
    if((entry->node[0] == '\0') ||
       (strncmp(entry->node,this_node,strlen(entry->node)) == 0) ||
       (strncmp(entry->node,this_node,strlen(this_node)) == 0)){
      if(entry->status != DEAD){
/* ///////////////////////////////////////////////////
// send message to the logbook
/////////////////////////////////////////////////// */
#ifndef VXWORKS
	add_err_string(entry->process_name);
	send_err_string(XPC_Self_strt);
#endif
/* //////////////////////////////////////////////////
// send message to other xpc's
////////////////////////////////////////////////// */
	send_other_xpc(entry->process_name,ALIVE);
      }
    }
    (*n_checkprg) = (*n_checkprg) + 1;
  }
}
/*******************************************************************/
void send_other_xpc(proc,status)
char *proc;
int status;
{
	char ms_buf[200],fnam[200],snd_node[100],snd_proc[100],junk[100];
	FILE *fp;

	sprintf(ms_buf,"Updat_Tbl %s %s %d",proc,this_node,status);
    
        if(decode_filename("$(USERUNIDAQ)/conf/nodefile",fnam) < 0){
          printf("Cannot decode USERUNIDAQ (/conf/nodefile)\n");
          return;
        }
        if((fp = fopen(fnam,"r")) == NULL){
          printf("Cannot open %s\n",fnam);
          return;
        }
        snd_node[0] = '\0';
        while(fscanf(fp,"%s",snd_node) > 0 ) {
	  fscanf(fp,"%s",junk);
	  if(strncmp(snd_node,this_node,strlen(snd_node))!= 0){ 
	    sprintf(snd_proc,"xpc@%s",snd_node);
	    Send_Message(snd_proc,ms_buf);
	  }
        }
        fclose(fp);

}
/************************************************************/
void No_Event()
{
#ifdef VXWORKS
  time_t tv;
#else
  struct timeval tv;
  struct timezone tz;
#endif
  Table *entry;
  short s;
  int i,ps;
	
/* //////////////////////////////////////////////////////////////////////
// fill in those processes that started up by users not using the XPC
////////////////////////////////////////////////////////////////////// */
  update_xpc_shm();
  check_process_changes();
#ifdef VXWORKS
  time(&tv);
  if((t_queuetest != 0) &&(((int)tv-t_queuetest) > 10)) {
#else
  gettimeofday(&tv,&tz);
  if((t_queuetest != 0) &&((tv.tv_sec-t_queuetest) > 10)) {
#endif
/************************************************************************/
/* Something terrible is going on and No messages are being sent at all */
/************************************************************************/
    check_all_queues(); 
  }
  for(i=0;i<*n_checkprg;i++) {
    entry = &(Common_Table[i]);
#ifdef VXWORKS
    if((tv-checkprg[i].t_last) > checkprg[i].t_msg) {
#else
    if((tv.tv_sec-checkprg[i].t_last.tv_sec) > checkprg[i].t_msg) {
#endif
      if(strcmp(entry->process_name,ownpname) == 0)
	Send_Message(ownpname,"set queuetest = 0");
      else
	Check_Process(i,entry);
#ifdef VXWORKS
      time(&tv);
#else
      gettimeofday(&tv,&tz);
#endif
      if(strcmp(entry->process_name,ownpname) == 0) {
#ifdef VXWORKS
	t_queuetest = tv;
      }
      checkprg[i].t_last = tv;
#else
	t_queuetest = tv.tv_sec;
      }
      memcpy(&(checkprg[i].t_last),&tv,sizeof(struct timeval));
#endif
    }
    else if(entry->status >= BUSY){
      read_value_from_sp(entry->process_name,"proc_status",&s,&ps);
      if(ps > BUSY) ps = BUSY;
      if(ps != entry->status){
	entry->status = ps;
	Write_to_Table(entry,i);
      }
    }
  }
  idle(1);
}
/***********************************************************/
check_process_changes() 
{
  int i;
#ifdef VXWORKS
  time_t tv;
#else
  struct timeval tv;
  struct timezone tz;
#endif
  Table *entry;

#ifdef VXWORKS
  time(&tv);
#else
  gettimeofday(&tv,&tz);
#endif
  for(i=0;i<*n_checkprg;i++) {
    entry = &(Common_Table[i]);
/* ///////////////////////////////////////////////////////////////////
// Has the status of the process changed ??
////////////////////////////////////////////////////////////////// */
    if(entry->status != checkprg[i].state) {
#ifdef VXWORKS
      checkprg[i].t_last = tv;
#else
      memcpy(&(checkprg[i].t_last),&tv,sizeof(struct timeval));
#endif
      checkprg[i].state = entry->status;
      checkprg[i].restart = NO;
      checkprg[i].no_check = NO;
      checkprg[i].t_msg = MSGTIME;
      if((entry->node[0] == '\0') ||
	 (strncmp(entry->node,this_node,strlen(entry->node)) == 0) ||
	 (strncmp(entry->node,this_node,strlen(this_node)) == 0)){
/* ///////////////////////////////////////////////////
// send message to the logbook
/////////////////////////////////////////////////// */
#ifndef VXWORKS
	add_err_string(entry->process_name);
	if(entry->status == ALIVE) send_err_string(XPC_Self_strt);
	if(entry->status == DEAD) send_err_string(XPC_Self_stop);
#endif
/* //////////////////////////////////////////////////
// send message to other xpc's
////////////////////////////////////////////////// */
	send_other_xpc(entry->process_name,entry->status);
      }
    }
  }
}
/***********************************************************/
void check_all_queues()
{
	int i,rtrn;
	Table entry;
#ifdef VXWORKS
	int count;
#else
	struct msqid_ds msbuf;
#endif

	for(i=0;i<*n_checkprg;i++) {
	  Read_from_Table(&entry,i);
	  if((entry.node[0] == '\0') ||
	     strncmp(entry.node,this_node,strlen(entry.node)) == 0) {
	    if(entry.status != DEAD) {
#ifdef VXWORKS
	      count = msgQNumMsgs(entry.msgq_id); 
	      if(count == ERROR || count >= MAX_MESSAGES) 
		Dead_Process(&entry,i); 
#else
	      rtrn = msgctl(entry.msgq_id,IPC_STAT,&msbuf);
	      if(rtrn == -1) Dead_Process(&entry,i);
	      if(msbuf.msg_qnum >= MAX_MESSAGES) Dead_Process(&entry,i);
	      if(msbuf.msg_cbytes >= MAX_MSQBYTES) Dead_Process(&entry,i);
#endif
	    }
	  }
	}
}
/***********************************************************/
void Check_Process(index,entry)
int index;
Table *entry;
{
  char *node,tmp[100];
  short stpe;
  int p_stat,ps,i;

/*** Do NOT check DEAD processes ***/
  if(entry->status == DEAD) return;

/*** Do NOT check processes running on a different node ***/
  node = entry->node;
  if(node[0] != '\0'){
    if((strncmp(node,this_node,strlen(this_node)) != 0) &&
       (strncmp(node,this_node,strlen(node)) != 0)) return;
  }

/*** Do NOT check the XPC (this process) ***/
  if(strcmp(entry->process_name,ownpname) == 0) return;

/*** Do NOT check processes that need to be ignored ***/
  if(checkprg[index].no_check == YES) return;

/*** Now the proposed six step check: ***/
/*** Step 1: Read the status from the status-path ***/
  read_value_from_sp(entry->process_name,"proc_status",&stpe,&p_stat);
/*** Step 2: Send a busy-signal ***/
  kill(entry->process_id,BUSY_SIGNAL);
/*** Step 3: Wait to a RESPONSE maximum, or until the status changed ***/
  for(i=0;i<RESPONSE;i++){
#ifdef VXWORKS
    taskDelay(sysClkRateGet());
#else
    sleep(1);
#endif
    read_value_from_sp(entry->process_name,"proc_status",&stpe,&ps);
    if(ps != p_stat) break;
  }
/*** Step 4: If the status did not change, perform steps 2 and 3 again ***/
  if(ps == p_stat){
    kill(entry->process_id,BUSY_SIGNAL);
    for(i=0;i<RESPONSE;i++){
#ifdef VXWORKS
      taskDelay(sysClkRateGet());
#else
      sleep(1);
#endif
      read_value_from_sp(entry->process_name,"proc_status",&stpe,&ps);
      if(ps != p_stat) break;
    }
  }
/*** Step 4 result: Still no change, the process is dead ***/
  if(ps == p_stat) {
    Dead_Process(entry,index);
    return;
  }
/*** Step 5: When status used to be ALIVE, set it again ***/
  if(p_stat == ALIVE){
    sprintf(tmp,"Set proc_status = %d",ALIVE);
    Send_Message(entry->process_name,tmp);
#ifdef VXWORKS
    taskDelay(sysClkRateGet());
#else
    sleep(1);
#endif
    read_value_from_sp(entry->process_name,"proc_status",&stpe,&p_stat);
  }
/*** Step 6: Set the appropriate status in the Status Path ***/
  if(p_stat >=BUSY) p_stat = BUSY;
  if(entry->status !=p_stat){
    entry->status = p_stat;
    Write_to_Table(entry,index);
  }
  return;
}
/***********************************************************/
void Dead_Process(entry,index)
Table *entry;
int index;
{
  char tmp[100];
  
#ifdef _DEBUG
  printf("React to a dead process\n");
#endif
/* //////////////////////////////////////////////////
// Check if messages can be sent to the XPC
////////////////////////////////////////////////// */
  if(strcmp(entry->process_name,ownpname) == 0) {
#ifndef VXWORKS
    send_err_string(XPC_Send);
#endif
    return;
  }
/* ///////////////////////////////////////////////////
// send message to the logbook
/////////////////////////////////////////////////// */
#ifndef VXWORKS
  add_err_string(entry->process_name);
  send_err_string(XPC_Dead);
#endif
/* //////////////////////////////////////////////////
// send message to other xpc's
////////////////////////////////////////////////// */
  send_other_xpc(entry->process_name,DEAD);
/* ////////////////////////////////////////////////
// clean up the table and stuff
/////////////////////////////////////////////// */
  if(entry->status != DEAD){
    kill(entry->process_id,9);
    remove_proc_from_sp(entry->process_name);
    Delete_MsgQ(entry->msgq_id);
    entry->status = DEAD;
    checkprg[index].state = DEAD;
    Write_to_Table(entry,index);
  }
  if(checkprg[index].restart == YES){
    sprintf(tmp,"YES -p %d %s",checkprg[index].t_msg,entry->process_name);
    Start_Process("xpc",tmp);
  }
}
/***********************************************************/
Start_Process(send,args)
char *send,*args;
{
  int i_restart,t_check;
  char ex_name[50],ex_node[100],ex_cmd[100],ps_name[50];
  int old_prg,index;
  char run_node[100];
#ifdef VXWORKS
  time_t tv,ts;
#else
  struct timeval tv,ts;
  struct timezone tz;
#endif
  Table tbl;
	
  get_start_args(args,&i_restart,&t_check,ex_name,ex_node,ex_cmd,ps_name);
#ifndef VXWORKS
  add_err_string(ps_name);
#endif
/*** Check if the process is already alive somewhere ***/
  if(search_process(ps_name,run_node) >=0) {
#ifndef VXWORKS
    send_err_string(XPC_NOstrt);
#endif
    return TRUE;
  }
/*** Check if this is the correct node ***/
  if((ex_node[0] == '\0') || 
     (strncmp(ex_node,this_node,strlen(ex_node)) == 0) ||
     (strncmp(ex_node,this_node,strlen(this_node)) == 0) ) {
    index = Search_Table(ps_name);
    if(index >= 0) old_prg = checkprg[index].restart;
    else           old_prg = NO;
/*** Execute the command ***/
#ifdef VXWORKS
    vxsystem(ex_cmd);
#else
    system(ex_cmd);
#endif
/*** Wait until the process is listed in the table ***/
    tbl.status = DEAD;
#ifdef VXWORKS
    time(&ts);
    tv = ts;
    while((tv-ts) < 5){
#else
    gettimeofday(&ts,&tz);
    tv.tv_sec = ts.tv_sec;
    while((tv.tv_sec-ts.tv_sec) < 5){
#endif
      if(index < 0) index = Search_Table(ps_name);
      if(index >= 0) {
	Read_from_Table(&tbl,index);
	if(tbl.status != DEAD) break;
      }
      idle(1);
#ifdef VXWORKS
      time(&tv);
#else
      gettimeofday(&tv,&tz);
#endif
    }
/*** fill the checkprg control structure  and send messages ***/
    if(tbl.status == DEAD) {
#ifndef VXWORKS
      send_err_string(XPC_NOstrt);
#endif
    }
    else {
      checkprg[index].restart = i_restart;
      checkprg[index].no_check = NO;
      checkprg[index].t_msg = t_check;
      checkprg[index].state = tbl.status;
      if(index >= *n_checkprg) *n_checkprg = index+1;
#ifdef VXWORKS
      checkprg[index].t_last=tv;
#else
      memcpy(&(checkprg[index].t_last),&tv,sizeof(struct timeval));
      if(old_prg == YES) send_err_string(XPC_REstrt);
      else               send_err_string(XPC_STRT);
#endif
      send_other_xpc(ps_name,ALIVE);
    }
  }
  else {
/*** send the stuff to the correct xpc ***/
    sprintf(ex_name,"xpc@%s",ex_node);
    sprintf(ex_cmd,"START %s",args);
    Send_Message(ex_name,ex_cmd);
  }
  return TRUE;
}
/***********************************************************/
get_start_args(line,i_restart,t_check,ex_name,ex_node,ex_cmd,ps_name)
char *line,*ex_name,*ex_node,*ex_cmd,*ps_name;
int *i_restart,*t_check;
{
  char word[50],tmp[150],rest[150];
  char *c;

/** First get the restart option ***/
  Parse(line,word,tmp);
  if(strcasecmp(word,"YES") == 0)     *i_restart = YES;
  else if(strcasecmp(word,"NO") == 0) *i_restart = NO;
  else   {
    *i_restart = NO;
    strcpy(tmp,line);
  }
/*** Rest of options are in "tmp". Now get the checking time ***/
  *t_check = MSGTIME;
  Parse(tmp,word,rest);
  if(strcmp(word,"-p") == 0) {
    if(sscanf(rest,"%d",t_check) != 1) {
      *t_check = MSGTIME;
      strcpy(tmp,rest);
    }
    else Parse(rest,word,tmp);
  }
/*** Rest of options are in "tmp" again. Next the name of the executable ***/
  Parse(tmp,ex_name,rest);
/*** And of course its node. Correct the name when needed ***/
  ex_node[0] = '\0';
  c = strrchr(ex_name,'@');
  if (c != NULL) {
    *c = '\0';
    c++;
    strcpy(ex_node,c);
  }
/*** The options are in "rest". Give the executable command ***/
  sprintf(ex_cmd,"%s %s &",ex_name,rest);
/*** find the process-name, first from the name of the executable ***/
  c = strrchr(ex_name,'/');
  if(c == NULL) strcpy(ps_name,ex_name);
  else {
    c+=1;
    strcpy(ps_name,c);
  }
/*** now from the options ***/
  c = strstr(rest,"-n ");
  if (c != NULL) {
    c+=2;
    if(sscanf(c,"%s",word) == 1) strcpy(ps_name,word);
  }
}
/***********************************************************/
int search_process(ps_name,node)
char *ps_name,*node;
{
  int index;
  Table entry;
  char nlist[50][100];
  int n_nodes;

  index = Search_Table(ps_name);
  if (index >= 0) {
    Read_from_Table(&entry,index);
    if((strncmp(entry.node,this_node,strlen(this_node)) == 0 ) ||
       (strncmp(entry.node,this_node,strlen(entry.node)) == 0 ) ||
       (entry.node[0] == '\0')) {
      if(entry.status != DEAD){
	strcpy(node,entry.node);
	return(1);
      }
    }
    else if(entry.status != DEAD){
      if(rpc_Check_Process(entry.node,ps_name) == 1){
	strcpy(node,entry.node);
	return(2);
      }
      else rpc_Update_Table(index,ownpname,entry.node,DEAD);
    }
  }
  rpc_get_nodes(nlist,100,&n_nodes);
  index = 0;
  for(index=0;index < n_nodes; index++){
    if((strncmp(this_node,nlist[index],strlen(nlist[index])) != 0) &&
       (strncmp(this_node,nlist[index],strlen(this_node)) != 0)){
      if(rpc_Check_Process(nlist[index],ps_name) == 1) {
	rpc_Update_Table(-1,ps_name,nlist[index],ALIVE);
	strcpy(node,nlist[index]);
	return(3);
      }
    }
  }
  return(-1);
}
/***********************************************************/
Stop_Process(header,s)
char *header,*s;
{
  char ps_name[100],ps_node[100],xpc_node[100];
  char *c;
  int i,index;
  Table tbl;

  if(sscanf(s,"%s",ps_name) <=0) return TRUE;
/*** send the message to a remote node when requested ***/
  if((c = strrchr(ps_name,'@')) != NULL) {
    *c = '\0';
    c++;
    if((strncmp(this_node,c,strlen(c)) != 0) &&
       (strncmp(this_node,c,strlen(this_node)) != 0) ){
      sprintf(xpc_node,"xpc@%s",c);
      sprintf(ps_name,"STOP %s",s);
      if((c = strchr(ps_name,'@'))!= NULL) *c = '\0';
      Send_Message(xpc_node,ps_name);
      return TRUE;
    }
  }
/*** Check where the process is running ***/
  i = search_process(ps_name,ps_node);
  if(i < 0) return TRUE;
/*** process is running remotely, send the message ***/
  if(i > 1){
    sprintf(xpc_node,"xpc@%s",ps_node);
    sprintf(ps_name,"STOP %s",s);
    Send_Message(xpc_node,ps_name);
    return TRUE;
  }
/*** STOP the local process ***/
  Send_Message(ps_name,"EXIT");
  if((index = Search_Table(ps_name)) < 0) return TRUE;
  for(i=0;i<5;i++) {
    Read_from_Table(&tbl,index);
    checkprg[index].state = tbl.status;
    if(tbl.status == DEAD) break;
#ifdef VXWORKS
    taskDelay(sysClkRateGet());
#else
    sleep(1);
#endif
  }
/*** send message to the logbook and xpc ***/
  if(tbl.status == DEAD) {
#ifndef VXWORKS
    add_err_string(ps_name);
    send_err_string(XPC_STOP);
#endif
    send_other_xpc(ps_name,DEAD);
  }
  return TRUE;
}
/***********************************************************/
Ignore_Process(header,args)
char *header,*args;
{
  int i_ignore = YES;
  char ps_name[100],ps_node[100],xpc_node[100],s[100],tmp[100];
  char *c;
  int index;

  Parse(args,tmp,s);
  if(sscanf(s,"%s",ps_name) != 1) strcpy(ps_name,header); 
  if(strcasecmp(tmp,"on") == 0)  i_ignore = YES;
  if(strcasecmp(tmp,"off") == 0) i_ignore = NO;
/*** send the message to a remote node when requested ***/
  if((c = strrchr(ps_name,'@')) != NULL) {
    *c = '\0';
    c++;
    if((strncmp(this_node,c,strlen(c)) != 0) &&
       (strncmp(this_node,c,strlen(this_node)) != 0) ){
      sprintf(xpc_node,"xpc@%s",c);
      sprintf(ps_name,"IGNORE %s",args);
      if((c = strchr(ps_name,'@'))!= NULL) *c = '\0';
      Send_Message(xpc_node,ps_name);
      return TRUE;
    }
  }
/*** Check where the process is running ***/
  index = search_process(ps_name,ps_node);
  if(index < 0) return TRUE;
/*** process is running remotely, send the message ***/
  if(index > 1){
    sprintf(xpc_node,"xpc@%s",ps_node);
    sprintf(ps_name,"IGNORE %s",args);
    Send_Message(xpc_node,ps_name);
    return TRUE;
  }
/*** Ignore the local process ***/
  if((index = Search_Table(ps_name)) < 0) return TRUE;
  checkprg[index].no_check = i_ignore;
  return TRUE;
}
/***********************************************************/
Interrupt_Process(header,args)
char *header,*args;
{
  char ps_name[100],ps_node[100],xpc_node[100];
  char *c;
  int index;
  Table tbl;

  if(sscanf(args,"%s",ps_name) != 1) return TRUE; 
/*** send the message to a remote node when requested ***/
  if((c = strrchr(ps_name,'@')) != NULL) {
    *c = '\0';
    c++;
    if((strncmp(this_node,c,strlen(c)) != 0) &&
       (strncmp(this_node,c,strlen(this_node)) != 0) ){
      sprintf(xpc_node,"xpc@%s",c);
      sprintf(ps_name,"INTERRUPT %s",args);
      if((c = strchr(ps_name,'@'))!= NULL) *c = '\0';
      Send_Message(xpc_node,ps_name);
      return TRUE;
    }
  }
/*** Check where the process is running ***/
  index = search_process(ps_name,ps_node);
  if(index < 0) return TRUE;
/*** process is running remotely, send the message ***/
  if(index > 1){
    sprintf(xpc_node,"xpc@%s",ps_node);
    sprintf(ps_name,"INTERRUPT %s",args);
    Send_Message(xpc_node,ps_name);
    return TRUE;
  }
/*** Interrupt the local process ***/
  if((index = Search_Table(ps_name)) < 0) return TRUE;
  Read_from_Table(&tbl,index);
  kill(tbl.process_id,SIGTERM);
  return TRUE;
}
/***********************************************************/
Update_Table(header,args)
char *header,*args;
{
  char proc[100],node[100];
  int stat,index;

  if(sscanf(args,"%s %s %d",proc,node,&stat) == 3) {
    index = Search_Table(proc);
    rpc_Update_Table(index,proc,node,stat);
  }
  return TRUE;
}
/***********************************************************/
Test_XPC(header,args)
char *header,*args;
{
  Send_Message(header,"ALIVE");
  return TRUE;
}
/***********************************************************/
Abort_Exit()
{
  return TRUE;
}
/**********************************************************************/
