/*///////////////////////////////////////////////////////////////////
//
// file:    xpc_checker.C
// author:  C. Timmermans
// version: 1.0
// date:    1/25/93
/////////////////////////////////////////////////////////////////////////*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include<string.h>
#include <unistd.h>
#include <string.h>
#include<stdio.h>
#include<time.h>
#include<sys/time.h>
#include<sys/types.h>
#include<sys/ipc.h>
#include<sys/shm.h>
#include<signal.h>
#include<stdlib.h>
#include"template.h"
#include"xpc_checker.h"
#include"sdc_pack_msg_c.h"



void Init_Process();
void No_Event();
void Dead_Process();
void Ping_Process();
void Startup();

#define YES 1
#define NO  0
#define MSGTIME 60          /* time between messages */
#define DEADTIME 80         /* xpc is dead           */
#define SENDTIME 5          /* sending the message every five seconds */
#define MAX_NODES 50

/*****************************************************/
/* some variables for general use                    */
/*****************************************************/

struct xpccheck{
char node[100];
struct timeval t_last;
};

int n_nodes;


struct xpccheck xpc_check[MAX_NODES];

extern char ownpname[];
extern char this_node[];
extern char get_name_from_args();

main(argc, argv)
int argc;
char **argv;
{
  char string[500];
  int result;
  
  if(strcmp(get_name_from_args(argc,argv),"xpc_checker") != NULL) {
    printf("Do not start the xpc_checker under a new name\n");
    exit(1);
  }
  if((result = Template_Init(get_name_from_args(argc,argv))) < 0){
    printf("XPC_checker is already alive\n");
    exit(1);
  }
  Init_Process();
  Template_Main(NULL,NULL,(void (*)()) No_Event);
}


/***********************************************************/
void Init_Process() /*perform startup operations*/
{
  char fnam[100],snd_node[100],junk[100];
  FILE *fp;

  if(decode_filename("$(USERUNIDAQ)/conf/nodefile",fnam) < 0){
    printf("Cannot decode USERUNIDAQ (/conf/nodefile)\n");
    return;
  }
  n_nodes = 0;
  if((fp = fopen(fnam,"r")) == NULL){
    printf("Cannot open %s\n",fnam);
    return;
  }
  snd_node[0] = '\0';
  while(fscanf(fp,"%s",snd_node) > 0 ) {
    fscanf(fp,"%s",junk);
    strcpy(xpc_check[n_nodes].node,snd_node);
    n_nodes++;
    sprintf(fnam,"xpc@%s",snd_node);
    Alive(fnam,"");
  }
  fclose(fp);
}
/********************************************************************/
void No_Event()
{
  struct timeval tv;
  struct timezone tz;
  int tdiff;
  int i;
	
  gettimeofday(&tv,&tz);
/*////////////////////////////////////////
// check if a response is needed
////////////////////////////////////////*/
  for(i=0;i<n_nodes;i++){
    tdiff = tv.tv_sec - xpc_check[i].t_last.tv_sec; 
    if(tdiff > DEADTIME)
      Dead_Process(i);
    else if(tdiff > MSGTIME) {
      tdiff -=MSGTIME;
      if((SENDTIME*(tdiff/SENDTIME)) == tdiff)
	Ping_Process(i);
    }
  }
  idle(1);
}
/***********************************************************/
void Dead_Process(i)
int i;
/*///////////////////////////////////////////////////
// React to a dead XPC
////////////////////////////////////////////////////*/
{
#ifdef _DEBUG
  printf("The XPC is dead\n");
#endif
/*//////////////////////////////////////////////////
// send this message to the logbook
//////////////////////////////////////////////////*/
  send_err_string(XPC_Chk_XPC_dead);
/*/////////////////////////////////////////////////
// Start the XPC
/////////////////////////////////////////////////*/
  Startup(xpc_check[i].node);
}
/***********************************************************/
void Ping_Process(i)
int i;
/*/////////////////////////////////////////////
// Signal the XPC
////////////////////////////////////////////*/
{
  char rcv[80];
  int result;
	
  sprintf(rcv,"xpc@%s",xpc_check[i].node); 
  result = Send_Message(rcv,"TEST_XPC");
  if(result < -1) {
    printf("Cannot reach node %s\n",xpc_check[i].node);
    Alive(rcv,"");
  }
}
/***********************************************************/
void Startup(nde)
char *nde;
/*/////////////////////////////////////////////
// Startup the XPC
/////////////////////////////////////////////*/
{
  char tmp[100];

  if((strncmp(nde,this_node,strlen(nde)) !=0) &&
     (strncmp(nde,this_node,strlen(this_node)) !=0) )
    rpc_Start_Process(nde,"xpc");
  else
    system("xpc &");
  sprintf(tmp,"xpc@%s",nde);
  Alive(tmp,"");
}
/***********************************************************/
Alive(xpc_node,args)
char *xpc_node,*args;
{
  int i;
  char *node;
  struct timeval t_last;
  struct timezone tz;
	
  if((node = strrchr(xpc_node,'@')) == NULL) return TRUE;
  node+=1;
  
  gettimeofday(&t_last,&tz);
  i = 0;
  for(i=0;i<n_nodes;i++){
    if((strncmp(xpc_check[i].node,node,sizeof(node)) == 0) ||
       (strncmp(xpc_check[i].node,node,sizeof(xpc_check[i].node)) == 0)){
      xpc_check[i].t_last.tv_sec = t_last.tv_sec;
      return TRUE;
    }
  }
  return TRUE;
}
/***********************************************************/

