/*---------------------------------------------------------------*/
/*                                                               */
/*      UNIdump.c                                                */
/*      Donald BRENNER                                           */
/*      Superconducting Super Collider Laboratory                */
/*      July 20, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <math.h>
#include "dump.h"
#include "template.h"

#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      10 
#define DEFAULT_SLEEP    1

#define  NTIMEOUT       0
#define  HEADER_LEN     6

int Process_Event();
void display_int();
void display_header(); 

struct   NOVAD *nd;
int *data, length;
FILE *fp = NULL;

int dump_status  = TRUE;
int dump_all = TRUE;
int dump_begin = NULL;
int dump_end = NULL;



/*(a+out) ------------------------------------------------- */
main(argc, argv)
int argc;
char * argv[];
/*(b+out) -------------------------------------------------
Procedure Name
Main - main procedure of recorder
Synopsis
Description
main(argc, argv) is the main procedure of the dump. It
calls the template with the addresses of the Process_Event()
procedures. It also calls the Template_Init.                           

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char name[NV_MAXNAME + 1];
   char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];


   if(Template_Init(get_name_from_args(argc,argv)) < 0){
     printf("Process of name %s already exists\n",
            get_name_from_args(argc,argv));
     printf("Please use -n to set a different name\n");
     exit(0);
   }

   UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
   if ((int)(nd = (struct NOVAD *)UNI_nova_open(name, nova_prio, 1)) 
                == -1) {
       printf("Error in opening nova...\n");
       Send_Message ( ownpname, "EXIT" );
   }
   Template_Main((int (*)())Process_Event, NULL, NULL);
}





/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - is used to dump the incoming data
Synopsis
Description
Process_Event() receives the buffer and display all or part of
the data. 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   int status;
   char name[100], num[20], temp[20];
   int run_num,i,pos = 0, pos_first = 0;
   int len,j;
   int char_rep[4];
   int k;
   double bit_cnt;
   int  bit_len;
   int ct;
   int last_data = -1;
   int num_data = 0;
   
   if ( dump_status != TRUE) 
      return (TRUE);
      
   status = UNI_nova_get(nd, &data, &length);
   
   /*
    * for dump all of the event 
    */            
   if (dump_all == TRUE) {
      if (status != 0) return (FALSE);
      printf("New Buffer-------------------------------------------------\n");      
      while (pos*4 < length) {
         if(data[pos] == 0) break;
         display_header(&data[pos]);
         len = data[pos];
         j = dump_begin;
         num_data = 0;
         for ( i=pos+6; i< pos+len; i++) {
            if (data[i] == last_data) num_data++;
            else num_data=0;
            if (i >= pos+len-5) num_data=0;
            if (num_data < 10) display_int(data[i],j);
            if (num_data == 10) printf("                    ....\n                    ....\n");
            j++;
            last_data = data[i];
         }

         printf("\n\n");
         pos += len;
      };
      /*printf("\nQQQQQQQ\n");*/
   }
   /*
    * only part of the event  
    */
   else {
      if (status != 0) return(FALSE);
      printf("New Buffer-------------------------------------------------\n");
      while (pos*4 < length) {
         if(data[pos] == 0) break;
         display_header(&data[pos]);
         j = dump_begin;
         len = data[pos];
         for ( i=pos+HEADER_LEN+dump_begin; (i<= pos+HEADER_LEN+dump_end) 
	                                 && (i<(pos+len)); i++) {
            if (data[i] == last_data) num_data++;
            else num_data=0;
            if (i >= pos+dump_end-5) num_data=0;
            if (num_data < 10) display_int(data[i],j);
            if (num_data == 10) printf("                    ....\n                    ....\n");
            j++;
            last_data = data[i];
         }
         printf("\n\n");
         pos += len;
      };
      /*printf("\nQQQQQQQ\n");*/
   }
   UNI_nova_put(nd, data,length, 0); 
   return (TRUE);
}





/*(a+out) ------------------------------------------------- */
int On(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Begin - processes a on command
Synopsis
Description
Begin(header, arg_string) processes a on command.

Examples:
on  -to display all the buffer
on 1stword lastword     
on 1 3  
                                 
Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   char temp1[100], temp2[100], temp3[100], temp4[100];
   int num = 0;
   
   dump_status = TRUE;
   dump_all = TRUE;
   Parse(arg_string, temp1, temp2);
   
   if (strcmp(temp1,"\0") != 0) {
       Parse(temp2, temp3, temp4);
       dump_begin = atoi(temp1);
       dump_end = atoi(temp3);
       if (dump_begin != 0) { 
          dump_begin = dump_begin - 1; 
          dump_end= dump_end -1; 
       }
       dump_all = FALSE;
       if (dump_begin >= dump_end) dump_end = dump_begin+1;
   }
   
   return TRUE;
}



/*(a+out) ------------------------------------------------- */
int Off(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
End - processes an off command
Synopsis
Description
End(header, arg_string) processes an off command.  Ends the 
displaying of the buffer.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   printf("End received");
   dump_status = FALSE;
   return TRUE;
}





/*(a+out) ------------------------------------------------- */
int ExitR(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Exit - processes an end command
Synopsis
Description
End(header, arg_string) processes an exit command.  Calls end to
close the file and then calls nova_close.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   UNI_nova_close(nd);
   return(Exit(ownpname, " "));
}

void display_int(x,j)
int x, j;
{
            
   int run_num,i,pos = 0, pos_first = 0;
   int len;
   int char_rep[4];
   int k;
   double bit_cnt;
   int  bit_len;
   int ct;
            
            
            bit_len = -1; 
            for ( k = 31; k>=0; k--) {
               if ( !((k+1) % 8) ) {
                  bit_len++;
                  bit_cnt=7;
                  char_rep[bit_len] = 0;
               }
               else
               bit_cnt--;
               if (x &(1 << k)) { 
#ifdef SGI
                  char_rep[bit_len] = char_rep[bit_len] + (int)pow(2,bit_cnt);
#else
                  char_rep[bit_len] = char_rep[bit_len] + (1<<(int)bit_cnt);
#endif
               } 

           }
                      bit_cnt=7;
           for (ct =0;ct <=4;ct++) {
              if ((char_rep[ct] < 32) || (char_rep[ct] > 125)) {
                   char_rep[ct] = 46;
              }
          }
          printf("%6d     %10d    %5d       %5d       %c%c%c%c   %08X\n",
                 j, x, x>>16, (x-((x>>16)<<16)), char_rep[0], char_rep[1],
                 char_rep[2], char_rep[3], x);
/*          printf("%6d\t\t%6d\t\t%0-4X\t\t\t\t%0-4X\t\t\t %c%c%c%c\t\t%0-8X\n", j, x, x>>16, (x-((x>>16)<<16)), char_rep[0],char_rep[1],char_rep[2],char_rep[3], x);  */



}

void display_header(bhead)
int *bhead;
{
       printf("Event Length = %d words\n",bhead[0]);
       printf("Event Type   = %d     Run Number = %d     Event Number = %d\n",
               bhead[1], bhead[2], bhead[3]);
       printf("Mode         = %d     Reserve One  = %d\n", bhead[4], bhead[5]);
       printf("Word Number    I*4      I*2(1st)    I*2(2nd)    ascii     hex\n");
}

