/********************************************************************/
/*                                                                  */
/*    Utility program, logview.c                                    */
/*    R. Ball, 4/12/93                                              */
/*    Display last 24 lines of log.bookYYMMDD file                  */
/*    Update at most every 15 seconds, but only when the file has   */
/*    changed.  If file for current date does not exist, display    */
/*    yesterday's file.                                             */
/*    Link with libtmplt.a                                          */
/*                                                                  */
/********************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include "logbook.h"

extern int decode_filename();
#define ENV_NAME "LGBK_LOC"

main()
{
  struct stat buf;
  struct tm *now;
  time_t nowtime, modtime;
  off_t  fsize = 0;
  int jret, yy, mm, dd, ss, tsec;
  int last_time = -1;
  int yys = 0;
  int mms = 0;
  int dds = 0;
  int yesterday = 0;
  char log_file[255], dated_file[255], message[255];
  char tmfile[255], *env_val;

/*
 * See if the environmental variable for the logbook file exists.  If it
 * does, use that as the logbook file.  If it does not, use that one hardwired
 * at installation.
 */
  if((env_val = getenv(ENV_NAME)) == NULL)  strcpy(tmfile,LOGBOOK);
  else                                      strcpy(tmfile,env_val);

  jret = decode_filename ( tmfile, log_file );

  /*  Loop forever, or until <ctrl-c>, periodically printing the tail end
   *  of the log.book file.
   */

  while(1) {
    nowtime = time(NULL);
    now = localtime ( &nowtime );
    strcpy (dated_file, log_file);
  /*
   *  Save the yy,mm,dd,ss of right now, and encode them into the file name
   *  which is to be found.
   */
    yy = now->tm_year;
    mm = now->tm_mon+1;
    dd = now->tm_mday;
    ss = now->tm_sec + 60*now->tm_min + 3600*now->tm_hour;

    sprintf (&dated_file[strlen(log_file)], "%d%02d%02d\0", now->tm_year,
       	now->tm_mon+1, now->tm_mday);
  /* 
   *  Get the file status.  If this fails via ENOENT, try one day back in
   *  time.
   */
    jret = stat(dated_file, &buf);
    if (jret == -1)  {
      if (yesterday >= 1) yesterday = 2;
      else                yesterday = 1;
      if (errno == ENOENT)  {
        yy = now->tm_year;
        mm = now->tm_mon + 1;
        dd = now->tm_mday - 1;
        if (dd == 0)  {
	  mm = mm-1;
	  if (mm == 0) {
	    mm=12;
	    yy=yy-1;
	  }
	  switch (mm) {
	    case 1:
	    case 3:
	    case 5:
	    case 7:
	    case 8:
	    case 10:
	    case 12:
	      dd=31;
	      break;
	    case 2:
	      dd=28;          /* ignore leap year */
	      break;
	    default:
	      dd=30;
	  }
        }                   /* end of backward-over-month-boundary coding  */

  /*  
   *  Re-encode for the file name, and try to 'stat' it.
   */
        sprintf (&dated_file[strlen(log_file)], "%d%02d%02d\0", yy, mm, dd);
        jret = stat(dated_file, &buf);
        if (jret == -1)  {
	  printf (" Requested log file %s\n (or today's either) not found\n", 
		dated_file);
	  break;
        }
      }
      else {
        printf (" Unknown error in accessing log file %s\n", dated_file);
	break;
      }
    }
    else {
      yesterday = 0;
    }

  /*
   *  A file was found.  Check the time of last modification.  If that time
   *  was more recent than the last time of display, then show the last
   *  24 lines of the file, else sleep for 15 seconds and do this all over
   *  again.
   *
   *  4/30/93 -- change to compare via file size (nfs disk/time sync problem)
   */
    if      (yesterday == 2)  last_time = 0;
    else if (yesterday == 1)  {
      last_time = -1;
    }
    else  {
      now = localtime ( &(buf.st_mtime) );
      tsec = now->tm_sec + 60*now->tm_min + 3600*now->tm_hour;
/*      if ((yys==now->tm_year) && (mms==((now->tm_mon)+1)) &&   */
/*	                       (dds==now->tm_mday)) {            */
/*        if (tsec > last_time)  last_time = -1;                 */
/*      }                                                        */
      if (buf.st_size == fsize) {
	last_time = 0;
      }
/*         Reuse the else from the "time" logic.                 */
      else  {
        last_time = -1;
      }
      fsize = buf.st_size;
    }

  /*
   *  The file is marked for display if it was last modified more recently
   *  than the most recent display, or if this is the first display cycle.
   *  If displayed, save the yy, mm, dd from the file, and the current ss.
   */
    if (last_time == -1) {
      if (yesterday == 0)  {
        printf(" - - - - - - - - - - - - - - - - - - - - - - - - - - -\n");
      }
      else {
	printf (" --> Displaying yesterday's log file <--\n");
      }
      strcpy (message, "tail -22 ");
      strcat (message, dated_file);
      system(message);
      last_time = ss;
      yys = yy;
      mms = mm;
      dds = dd;
    }

  /*
   *  Wait before attempting the next display cycle.
   */
    sleep(15);
  }
}
